using System;
using System.Collections.Generic;
using System.Reflection;
using WideStream.Core.Runtime;
using WideStream.Startup.Resources;

namespace WideStream.Configuration.ComponentModel
{
    public static class EnumExtension
    {
        public static IList<ValuePair<String,Int32>> GetDisplayNames(this Enum enumeration)
        {
            var list = new List<ValuePair<String, Int32>>();

            foreach (FieldInfo fi in enumeration.GetType().GetFields(BindingFlags.Static | BindingFlags.Public))
            {
                Attribute attr;
                if ((attr = Attribute.GetCustomAttribute(fi, typeof(EnumNameAttribute))) != null)
                    list.Add(new ValuePair<String,Int32>(StringTable.GetString(attr.ToString()), (Int32)fi.GetValue(null)));
            }

            return list;
        }


        public static string ToDisplayString(this Enum enumeration)
        {
            FieldInfo fi = enumeration.GetType().GetField(
                enumeration.ToString(), BindingFlags.Static | BindingFlags.Public);

			if (fi == null)
				return String.Empty;
			else
			{
				Attribute attr;
				return (attr = Attribute.GetCustomAttribute(fi, typeof(EnumNameAttribute))) != null ?
					StringTable.GetString(attr.ToString()) : enumeration.ToString();
			}
        }


		public static bool HasDisplayName(this Enum enumeration)
		{
			FieldInfo fi = enumeration.GetType().GetField(
				enumeration.ToString(), BindingFlags.Static | BindingFlags.Public);
			return Attribute.GetCustomAttribute(fi, typeof(EnumNameAttribute)) != null;
		}
    }
}