﻿using System;
using System.Collections.Generic;

namespace WideStream.Configuration.ComponentModel
{
	public abstract class Options : ICloneable
	{
		#region Construction
        private List<String> enabledValues;

		protected Options()
		{
            enabledValues = new List<String>();
		}
		#endregion


		#region Methods
		object ICloneable.Clone()
		{
			return MemberwiseClone();
		}


		public Options Clone()
		{
			Options opt = (Options)MemberwiseClone();
            opt.enabledValues = new List<String>();
            return opt;
		}


        public bool AddEnabledValue(string value)
        {
            bool ret;

            if (!(ret = enabledValues.Contains(value)))
                enabledValues.Add(value);

            return !ret;
        }


        public bool RemoveEnabledValue(string value)
        {
            return enabledValues.Remove(value);
        }


        public string[] ObtainEnabledValues()
        {
            return enabledValues.ToArray();
        }


        public bool IsEnabled(string value)
        {
            return enabledValues.Contains(value);
        }


        public IEnumerable<String> IterateEnableValues()
        {
            return enabledValues;
        }


        public void ReadEnabledValues(string arr)
        {
            enabledValues.Clear();

            if (!String.IsNullOrEmpty(arr))
            {
                string[] vals = arr.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
                enabledValues.AddRange(vals);
            }
        }
		#endregion
    }
}
