﻿using System;
using System.Drawing;
using System.Windows.Forms;
using WideStream.Workspace;
using WideStream.Workspace.ComponentModel;

namespace WideStream.Configuration
{
    public static class EnvironmentViewState
    {
        #region Methods
        public static void SaveState(Main form)
        {
			RecoveryOptions conf = Config.Query<RecoveryOptions>();
            int state = (Int32)form.WindowState;
            
            if ( (conf.WindowStateSave == WindowStateSaveOption.OnlyMaximized && form.WindowState == FormWindowState.Minimized) ||
            (conf.WindowStateSave == WindowStateSaveOption.OnlyMinimized && form.WindowState == FormWindowState.Maximized) ||
            (conf.WindowStateSave == WindowStateSaveOption.Never && form.WindowState != FormWindowState.Normal) )            
                state = (Int32)FormWindowState.Normal;
                
            ApplicationRegistry.SetValue("Form.WindowState", state);
           
            if (form.WindowState != FormWindowState.Maximized && form.WindowState != FormWindowState.Minimized)
            {
                ApplicationRegistry.SetValue("Form.Location.X", form.Location.X);
                ApplicationRegistry.SetValue("Form.Location.Y", form.Location.Y);
                ApplicationRegistry.SetValue("Form.Width", form.Width);
                ApplicationRegistry.SetValue("Form.Height", form.Height);
            }

            string position = "Top";

            if (form.ToolContainer.LeftToolStripPanel.Contains(form.ToolBar))
                position = "Left";
            else if (form.ToolContainer.RightToolStripPanel.Contains(form.ToolBar))
                position = "Right";
            else if (form.ToolContainer.BottomToolStripPanel.Contains(form.ToolBar))
                position = "Bottom";

            ApplicationRegistry.SetValue("Form.ToolBar.Dock", position);
            ApplicationRegistry.SetValue("Form.ToolBar.Y", form.ToolBar.Top);
            ApplicationRegistry.SetValue("Form.ToolBar.X", form.ToolBar.Left);
            
            position = "Top";

            if (form.ToolContainer.LeftToolStripPanel.Contains(form.MainMenuStrip))
                position = "Left";
            else if (form.ToolContainer.RightToolStripPanel.Contains(form.MainMenuStrip))
                position = "Right";
            else if (form.ToolContainer.BottomToolStripPanel.Contains(form.MainMenuStrip))
                position = "Bottom";
            
            ApplicationRegistry.SetValue("Form.Menu.Dock", position);
            ApplicationRegistry.SetValue("Form.Menu.Y", form.MainMenuStrip.Top);
            ApplicationRegistry.SetValue("Form.Menu.X", form.MainMenuStrip.Left);

            ApplicationRegistry.SetValue("Form.CurrentPage", (Int32)form.CurrentPage.PageType);
			ApplicationRegistry.SetValue("Form.SplitterDistance", form.SplitterDistance);
            SaveGridState(form.LogView, "Form");

            foreach (IPage p in form.Pages.Values)
                SaveState(p);
        }


        public static void LoadState(Main form)
        {
			form.Visible = false;
            form.Width = ApplicationRegistry.GetValue<Int32>("Form.Width", form.Width);
            form.Height = ApplicationRegistry.GetValue<Int32>("Form.Height", form.Height);
            form.WindowState = (FormWindowState)ApplicationRegistry.GetValue<Int32>("Form.WindowState", (Int32)form.WindowState);
            form.ChangeCurrentPage((Page)ApplicationRegistry.GetValue<Int32>("Form.CurrentPage", (Int32)Page.DownloadManager));
			form.SplitterDistance = ApplicationRegistry.GetValue<Int32>("Form.SplitterDistance", form.SplitterDistance);
            form.ToolBar.Top = ApplicationRegistry.GetValue<Int32>("Form.ToolBar.Y", form.ToolBar.Top);
            form.ToolBar.Left = ApplicationRegistry.GetValue<Int32>("Form.ToolBar.X", form.ToolBar.Left);
            SetFormLocation(form);
            
            string position = ApplicationRegistry.GetValue<String>("Form.ToolBar.Dock", "Top");
            
            if (position == "Left")
                form.ToolContainer.LeftToolStripPanel.Controls.Add(form.ToolBar);
            else if (position == "Right")
                form.ToolContainer.RightToolStripPanel.Controls.Add(form.ToolBar);
            else if (position == "Bottom")
                form.ToolContainer.BottomToolStripPanel.Controls.Add(form.ToolBar);

            form.MainMenuStrip.Top = ApplicationRegistry.GetValue<Int32>("Form.Menu.Y", form.MainMenuStrip.Top);
            form.MainMenuStrip.Left = ApplicationRegistry.GetValue<Int32>("Form.Menu.X", form.MainMenuStrip.Left);
            position = ApplicationRegistry.GetValue<String>("Form.Menu.Dock", "Top");

            if (position == "Left")
                form.ToolContainer.LeftToolStripPanel.Controls.Add(form.MainMenuStrip);
            else if (position == "Right")
                form.ToolContainer.RightToolStripPanel.Controls.Add(form.MainMenuStrip);
            else if (position == "Bottom")
                form.ToolContainer.BottomToolStripPanel.Controls.Add(form.MainMenuStrip);

            form.Visible = true;
        }


        public static void SetFormLocation(Form form)
        {
			RecoveryOptions conf = Config.Query<RecoveryOptions>();
            if (conf.WindowPositionSave == WindowPositionSaveOption.AlwaysCentered)
                form.Location = new Point((Screen.PrimaryScreen.WorkingArea.Width - form.Width) / 2,
                    (Screen.PrimaryScreen.WorkingArea.Height - form.Height) / 2);
            else if (conf.WindowPositionSave == WindowPositionSaveOption.WindowsControlled)
                form.StartPosition = FormStartPosition.WindowsDefaultLocation;
            else if (conf.WindowPositionSave == WindowPositionSaveOption.Save)
                form.Location = new Point(ApplicationRegistry.GetValue<Int32>("Form.Location.X", form.Location.X),
                    ApplicationRegistry.GetValue<Int32>("Form.Location.Y", form.Location.Y));

        }


        private static void LoadGridState(DataGridView gridView, string key)
        {
            foreach (DataGridViewColumn c in gridView.Columns)
            {
                c.Width = ApplicationRegistry.GetValue<Int32>(String.Format("{0}.{1}.Width", key, c.Name), c.Width);
                c.DisplayIndex = ApplicationRegistry.GetValue<Int32>(String.Format("{0}.{1}.DisplayIndex", key, c.Name), c.DisplayIndex);
            }
        }


        private static void SaveGridState(DataGridView gridView, string key)
        {
            foreach (DataGridViewColumn c in gridView.Columns)
            {
                ApplicationRegistry.SetValue(String.Format("{0}.{1}.Width", key, c.Name), c.Width);
                ApplicationRegistry.SetValue(String.Format("{0}.{1}.DisplayIndex", key, c.Name), c.DisplayIndex);
            }
        }


        public static void LoadState(IPage page)
        {
            if (page.DisplayControl != null)
                LoadGridState(page.DisplayControl, page.PageType.ToString());
        }



        private static void SaveState(IPage page)
        {
            if (page.DisplayControl != null)
                SaveGridState(page.DisplayControl, page.PageType.ToString());
        }
        #endregion
    }
}
