﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using WideStream.Core.Runtime;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.DownloadManager.Network;
using WideStream.DownloadManager.Configuration;
using WideStream.Workspace.Forms;
using WideStream.Startup.Resources;

namespace WideStream.Configuration.Presentation
{
    public partial class AddinsDialog : Form
    {
        public AddinsDialog()
        {
            Properties = new List<AddinProperty>();
            InitializeComponent();
            itemList.DefaultImage = Image.FromStream(typeof(AddinsDialog).Assembly.
                GetManifestResourceStream("WideStream.Images.web.bmp"));

            typeCombo.Items.Clear();
            typeCombo.Items.Add(GetString("Protocols"));
            typeCombo.Items.Add(GetString("Adapters"));
        }

        private void typeCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            itemList.Items.Clear();
            
            if (typeCombo.SelectedIndex == 0)
            {
                foreach (DownloadProtocol t in DownloadProtocolManager.EnumerateDownloadTypes())
                    itemList.Items.Add(new IconListEntry { Text = t.Name, Value = t });

                Properties.Clear();
                Properties.Add(new AddinProperty { Name = GetString("Protocol"), GetValueCallBack =
                    () => Sel<DownloadProtocol>().Name
                });
                Properties.Add(new AddinProperty { Name = GetString("Description"), GetValueCallBack =
                    () => Sel<DownloadProtocol>().Description
                });
            }
            else if (typeCombo.SelectedIndex == 1)
            {
                foreach (DownloadAdapterAddin a in DownloadAdapterManager.EnumerateAdapters())
                    itemList.Items.Add(new IconListEntry { Text = a.Name, Icon = a.Image, Value = a });
                AdapterManagerOptions opt = Config.Query<AdapterManagerOptions>();

                Properties.Clear();
                Properties.Add(new AddinProperty { Name = GetString("Adapter"), GetValueCallBack = 
                    () => Sel<DownloadAdapterAddin>().Name });
                Properties.Add(new AddinProperty { Name = GetString("MatchPattern"), GetValueCallBack = 
                    () => Sel<DownloadAdapterAddin>().MatchPattern });
                Properties.Add(new AddinProperty { Name = GetString("Description"), GetValueCallBack = 
                    () => Sel<DownloadAdapterAddin>().Description });
                Properties.Add(new AddinProperty { Name = GetString("CurrentState"), GetValueCallBack =
                    () => !opt.DisabledAdapters.Contains(Sel<DownloadAdapterAddin>().Key),
                    EnableCallBack = (b) => 
                    {
                        if (b)
                            opt.DisabledAdapters.Add(Sel<DownloadAdapterAddin>().Key); 
                        else
                            opt.DisabledAdapters.Remove(Sel<DownloadAdapterAddin>().Key);
                        return true;
                    } 
                });
            }

            if (itemList.Items.Count > 0)
                itemList.SelectedIndex = 0;
        }

        private void AddinsDialog_Load(object sender, EventArgs e)
        {
            Text = GetString("DialogTitle");
            closeButton.Text = GetString("CloseCaption");
            typeCombo.SelectedIndex = 0;
            itemList.Focus();
        }


        internal T Sel<T>()
        {
            return ((IconListEntry)itemList.SelectedItem).Value.DirectCast<T>();
        }


        internal List<AddinProperty> Properties { get; private set; }

        private void itemList_SelectedIndexChanged(object sender, EventArgs e)
        {
            panel.Controls.Clear();

            foreach (AddinProperty p in Properties)
            {
                Label l = new Label();
                l.Font = new Font(l.Font, FontStyle.Bold);
                l.Text = p.Name + ":";
                l.AutoSize = true;
                panel.Controls.Add(l);
                panel.SetFlowBreak(l, true);
           
                if (p.EnableCallBack != null)
                {
                    LinkLabel ll = new LinkLabel();
                    ll.AutoSize = true;
                    ll.Text = (Boolean)p.GetValueCallBack() ? GetString("Disable") : GetString("Enable");
                    ll.Click += (o, ev) => 
                    { 
                        p.EnableCallBack((Boolean)p.GetValueCallBack()); 
                        ll.Text = (Boolean)p.GetValueCallBack() ? GetString("Disable") : GetString("Enable"); 
                    };
                    panel.Controls.Add(ll);
                    panel.SetFlowBreak(ll, true);                    
                }
                else
                {
                    Label lt = new Label();
                    lt.Text = (String)p.GetValueCallBack();
                    lt.AutoSize = true;
                    panel.Controls.Add(lt);
                    panel.SetFlowBreak(lt, true);                    
                }

                Panel pp = new Panel();
                pp.Height = 5;
                panel.Controls.Add(pp);
                panel.SetFlowBreak(pp, true);
            }
        }


        private string GetString(string key)
        {
            return StringTable.GetString(String.Format("Addins.{0}", key));
        }
    }
}
