﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using WideStream.DownloadManager;
using WideStream.DownloadManager.Presentation;
using WideStream.FtpManager;
using WideStream.FtpManager.Presentation;
using WideStream.Workspace.Configuration;
using WideStream.Configuration.ComponentModel;
using System.Reflection;
using WideStream.Startup.Resources;

namespace WideStream.Configuration.Presentation
{
    public partial class ConfigurationDialog : Form
    {
        #region Construction
        public ConfigurationDialog()
        {
            InitializeComponent();
			Renderer = new OptionPageRenderer();
        }
        #endregion


		#region Methods
		private void ConfigurationDialog_Load(object sender, EventArgs e)
		{
			string lastCategory = String.Empty;
            int displayIndex = 0;

			for (int i = 0; i < Config.RegisteredOptions.Count; i++)
			{
				RegistrationInfo inf = Config.RegisteredOptions[i];

                if (inf.Hidden)
                    continue;

                string cat = StringTable.GetString(inf.Category);

                if (lastCategory != cat)
				{
					pageBox.Items.Add(String.Format(">{0}", cat));

                    if (String.Compare(inf.Category, DisplayCategory, true) == 0)
                        displayIndex = pageBox.Items.Count - 1;

                    lastCategory = cat;
				}

				pageBox.Items.Add(String.Format("{0}|{1}", StringTable.GetString(inf.ShortName), i));
			}

            pageBox.SelectedIndex = displayIndex;
            Text = GetString("DialogTitle");
            acceptButton.Text = GetString("AcceptTitle");
            cancelButton.Text = GetString("CancelTitle");
		}


		private void pageBox_SelectedIndexChanged(object sender, EventArgs e)
        {
			int index = pageBox.GetSelectedValue();

			if (index != -1)
			{
				controlPanel.Controls.Clear();
				RegistrationInfo inf = Config.RegisteredOptions[index];
				Options opt = Config.Query(inf.Id);
				Control page = Renderer.RenderOptions(opt);
				controlPanel.Controls.Add(page);
			}
        }


		private void acceptButton_Click(object sender, EventArgs e)
		{
            if (Renderer.ApplyChanges())
            {
                MessageBox.Show(this, GetString("RequiresRestart"), ProductName,
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
			
            Config.Update();
		}


        private string GetString(string key)
        {
            return StringTable.GetString(String.Format("Options.{0}", key));
        }
		#endregion
		

		#region Properties
		public OptionPageRenderer Renderer { get; private set; }

        public string DisplayCategory { get; set; }
		#endregion
	}
}
