﻿using System;
using WideStream.Startup.Resources;

namespace WideStream.Core.Network
{
	public struct DataSize
	{
		#region Construction
		private long size;
		
		public DataSize(long size)
		{
			this.size = size;
		}
		#endregion


		#region Methods
		public override string ToString()
		{
			return ToString(SizeFormat.Regular);
		}


		public string ToString(SizeFormat format)
		{
			SizeMeasure measure = default(SizeMeasure);

			if (GigaBytes >= 1)
				measure = SizeMeasure.GigaBytes;
			else if (MegaBytes >= 1)
				measure = SizeMeasure.MegaBytes;
			else if (KiloBytes >= 1)
				measure = SizeMeasure.KiloBytes;
			else
				measure = SizeMeasure.Bytes;

			return ToString(measure, format);
		}


		public string ToString(SizeMeasure measure)
		{
			return ToString(measure, SizeFormat.Regular);
		}


		public string ToString(SizeMeasure measure, SizeFormat format)
		{
			if (size == Int64.MaxValue)
				return StringTable.GetString("DataSize.Unknown");

            string formatKey = format == SizeFormat.Regular ? "Sizes" :
                "Speeds";
            string stringKey = format == SizeFormat.Regular ? "F" : "F0";
            string[] formats = StringTable.GetString(String.Format("DataSize.{0}", formatKey)).
                Split(new char[] { ';' }, StringSplitOptions.RemoveEmptyEntries); 
			string dataFormat = String.Empty;

			switch (measure)
			{
				case SizeMeasure.Bytes :
					dataFormat = Bytes.ToString();
					break;
				case SizeMeasure.KiloBytes:
                    dataFormat = KiloBytes.ToString(stringKey);
					break;
				case SizeMeasure.MegaBytes:
                    dataFormat = MegaBytes.ToString(stringKey);
					break;
				case SizeMeasure.GigaBytes:
                    dataFormat = GigaBytes.ToString(stringKey);
					break;
			}

			return String.Format("{0} {1}", dataFormat,
				formats[(Int32)measure]);
		}
		#endregion


		#region Operators
		public static implicit operator DataSize(long size)
		{
			return new DataSize(size);
		}


		public static implicit operator DataSize(int size)
		{
			return new DataSize(size);
		}


		public static implicit operator Int64(DataSize size)
		{
			return size.size;
		}


		public static explicit operator Int32(DataSize size)
		{
			return (Int32)size.size;            
		}
		#endregion


		#region Properties
		public long Bytes
		{
			get { return size; }
		}


		public float KiloBytes
		{
			get { return Bytes / 1024; }
		}


		public float MegaBytes
		{
			get { return KiloBytes / 1024; }
		}


		public float GigaBytes
		{
			get { return MegaBytes / 1024; }
		}
		#endregion
	}
}
