﻿using System;
using System.Xml;
using System.Linq;
using System.Linq.Expressions;
using System.Collections.Generic;
using WideStream.Configuration;
using System.IO;
using WideStream.DownloadManager.Presentation;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.Core.Network;
using WideStream.Configuration.ComponentModel;

namespace WideStream.DownloadManager.Configuration
{
    [Options(Category = "DownloadProcessOptions.Category", LongName = "DownloadProcessOptions.LongName", ShortName = "DownloadProcessOptions.ShortName")]
    public sealed class DownloadProcessOptions : Options
    {
        #region Construction
        private object syncRoot = new Object();

		public DownloadProcessOptions()
        {
			ErrorLimit = ErrorLimit.Error500;
            RetryTime = RetryTime.Sec3;
            Timeout = Timeout.Sec30;
            MaximumDownloadConnections = IntRange.N8;
            Priority = DownloadPriority.Normal;          
        }
        #endregion


		#region Events
		public event EventHandler PriorityChanged;
		internal void OnPriorityChanged()
		{
			EventHandler h = PriorityChanged;

			if (h != null)
				h(this, EventArgs.Empty);
		}


        public event EventHandler MaximumConnectionsChanged;
        private void OnMaximumConnectionsChanged()
        {
            EventHandler h = MaximumConnectionsChanged;

            if (h != null)
                h(this, EventArgs.Empty);
        }
		#endregion


		#region Properties
		private string _downloadPath;
        [Editor("DownloadProcessOptions.DownloadPath", ColSpan = true, Editor = PropertyEditor.FolderPicker)]
		public string DownloadPath 
        {
            get
            {
                if (String.IsNullOrEmpty(_downloadPath))
                {
                    lock (syncRoot)
                    {
                        if (String.IsNullOrEmpty(_downloadPath))
                        {
                            DirectoryInfo dir = Config.GetDefaultDirectory();
                            _downloadPath = dir.GetDirectories()[0].FullName;
							Config.EnsureExists(dir);
                        }
                    }
                }

                return _downloadPath;
            }
            set { _downloadPath = value; }
        }


        private IntRange _maximumDownloadConnectios;
        [Editor("DownloadProcessOptions.MaximumDownloadConnections")]
		public IntRange MaximumDownloadConnections 
        {
            get { return _maximumDownloadConnectios; }
            set 
            {
                if (_maximumDownloadConnectios != value)
                {
                    _maximumDownloadConnectios = value;
                    OnMaximumConnectionsChanged();
                }
            }
        }


		private DownloadPriority _priority;
        [Editor("DownloadProcessOptions.Priority")]
		public DownloadPriority Priority
		{
			get { return _priority; }
			set
			{
				if (_priority != value)
				{
					_priority = value;
					OnPriorityChanged();
				}
			}
		}

        [Editor("DownloadProcessOptions.Timeout")]
        public Timeout Timeout { get; set; }

        [Editor("DownloadProcessOptions.ErrorLimit")]
        public ErrorLimit ErrorLimit { get; set; }

        [Editor("DownloadProcessOptions.RetryTime")]
        public RetryTime RetryTime { get; set; }

        [Editor("DownloadProcessOptions.ManualStart")]
		public bool ManualStart { get; set; }
		#endregion
    }
}
