﻿using System;
using System.Xml;
using System.Linq;
using System.Linq.Expressions;
using System.Collections.Generic;
using WideStream.Configuration;
using System.IO;
using WideStream.DownloadManager.Presentation;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.Core.Network;
using WideStream.Configuration.ComponentModel;

namespace WideStream.DownloadManager.Configuration
{
    [Options(Category = "QueueManagerOptions.Category", LongName = "QueueManagerOptions.LongName", ShortName = "QueueManagerOptions.ShortName")]
	public sealed class QueueManagerOptions : Options
	{
		#region Construction
		private object syncRoot = new Object();

		public QueueManagerOptions()
		{
			TrafficUsage = TrafficUsage.Full;
			MaximumDownloads = IntRange.N6;
			AutoSaveInterval = AutoSaveInterval.Minute5;
			AccelerateFileSize = FileSize.MB10;
            ExistingFileAction = ExistingFileAction.ShowSelectDialog;
            FailOnCertificateErrors = CertificateErrors.None;
		}
		#endregion


        #region Methods
        public string GetDefaultQueueDefinitionPath()
        {
            DirectoryInfo dir = Config.GetDefaultDirectory();            
            Config.EnsureExists(dir);
            return Path.Combine(dir.FullName, "widestream_queue.xml");
        }
        #endregion


        #region Properties
        private string _queueDefinitionPath;
		public string QueueDefinitionPath
		{
			get
			{
				if (String.IsNullOrEmpty(_queueDefinitionPath))
				{
					lock (syncRoot)
					{
						if (String.IsNullOrEmpty(_queueDefinitionPath))
						    _queueDefinitionPath = GetDefaultQueueDefinitionPath();
					}
				}

				return _queueDefinitionPath;
			}
			set { _queueDefinitionPath = value; }
		}

        [Editor("QueueManagerOptions.AutoSaveInterval")]
		public AutoSaveInterval AutoSaveInterval { get; set; }

        [Editor("QueueManagerOptions.TrafficUsage")]
		public TrafficUsage TrafficUsage { get; set; }

        [Editor("QueueManagerOptions.SuspendQueue")]
		public bool SuspendQueue { get; set; }

        [Editor("QueueManagerOptions.MaximumDownloads")]
		public IntRange MaximumDownloads { get; set; }

        [Editor("QueueManagerOptions.AccelerateFileSize")]
		public FileSize AccelerateFileSize { get; set; }

        [Editor("QueueManagerOptions.AutoRestartFailed")]
		public bool AutoRestartFailed { get; set; }

        [Editor("QueueManagerOptions.ExistingFileAction")]
        public ExistingFileAction ExistingFileAction { get; set; }

        [Editor("QueueManagerOptions.RemoveFile")]
        public RemoveFileCondition RemoveFile { get; set; }

        [Editor("QueueManagerOptions.FailOnCertificateErrors", ColSpan = true, Editor = PropertyEditor.FlagsEditor, Height = 50)]
        public CertificateErrors FailOnCertificateErrors { get; set; }
		#endregion
	}
}
