﻿using System;
using System.Linq;
using System.Net;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.DownloadManager.Network.Protocols.Http;
using WideStream.Configuration;

namespace WideStream.DownloadManager.Network.Adapters
{
    public class DefaultAdapter : IDownloadAdapter
    {
        #region Construction
        public DefaultAdapter()
        {
            
        }
        #endregion


        #region Methods
        public virtual void Initialize(IDownloadProcess process)
        {
            Process = process;
        }


        public virtual AdapterResult Analyze(IConnectionInformation information, IDownloadResponse response)
        {
            HttpCookies cookies = ((HttpHeaders)information).Cookies;
            CookieString = cookies != null ? cookies.ToString() : null;
            RequiresRestart = cookies != null && cookies.Count > 0;
            return AdapterResult.Default;
        }


        public virtual AdapterResult Negotiate(IDownloadRequest request) 
        {
            if (!String.IsNullOrEmpty(CookieString))
            {
                HttpDownloadRequest req = (HttpDownloadRequest)request;
                req.Request.Headers.Add("Cookie", CookieString);
                RequiresRestart = false;
            }

            return AdapterResult.Default;
        }
        #endregion


        #region Properties
        public bool RequiresRestart { get; protected set; }

        public virtual Uri DiscoveryUrl
        {
            get { return Process.Properties.Url; }
        }


        public virtual Uri DownloadUrl
        { 
            get { return Process.Properties.Url; } 
        }


        public virtual string DiscoveryMethod
        {
            get
            {
                return ((HttpOptions)Process.Properties.ProtocolOptions.First(o => o is HttpOptions)).SendGet ?
                    WebRequestMethods.Http.Get : WebRequestMethods.Http.Head;
            }
        }


        public virtual int? MaximumConnections
        {
            get { return null; }
        }

        public IDownloadProcess Process { get; protected set; }

        public string CookieString { get; protected set; }
        #endregion
    }
}
