﻿using System;
using System.IO;
using WideStream.Core.Runtime;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.DownloadManager.Network.Protocols.Http;
using System.Net;

namespace WideStream.DownloadManager.Network.Adapters
{
    public sealed class DoujinstyleAdapter : IDownloadAdapter
    {
        #region Construction
        private const string HOST = "*doujinstyle.com";
        private const string SESSID = "PHPSESSID";
        private const string CTOKEN = "ctoken";

        public DoujinstyleAdapter()
        {
            
        }
        #endregion


        #region Methods
        public void Initialize(IDownloadProcess process)
        {
            Process = process;
        }


        public AdapterResult Analyze(IConnectionInformation information, IDownloadResponse response)
        {
            HttpCookies cookies = ((HttpHeaders)information).Cookies;

            if (cookies != null)
            {
                string sessId = cookies.GetCookie(SESSID);
                string ctok = cookies.GetCookie(CTOKEN);

                if (!String.IsNullOrEmpty(sessId) && !String.IsNullOrEmpty(ctok) &&
                    SessionId == null && Ctoken == null)
                {
                    SessionId = sessId;
                    Ctoken = ctok;

                    if (DownloadUrl == null)
                        RequiresRestart = (DownloadUrl = AnalyzeResponse(response)) != null;
                }
            }

            return AdapterResult.Default;
        }


        private Uri AnalyzeResponse(IDownloadResponse response)
        {
            HttpDownloadResponse resp = (HttpDownloadResponse)response;

            using (Stream stream = resp.Response.GetResponseStream())
            using (StreamReader sr = new StreamReader(stream))
            {
                string source = sr.ReadToEnd();
                UrlSpider spider = new UrlSpider("http");

                foreach (Uri uri in spider.Parse(response.ResponseUrl, source))
                    if (uri.Host.Like(HOST))
                        return new Uri(uri.ToString().Replace("&amp;", "&"));
            }

            return null;
        }

       
        public AdapterResult Negotiate(IDownloadRequest request)
        {
            if (!String.IsNullOrEmpty(SESSID) && !String.IsNullOrEmpty(CTOKEN))
            {
                HttpDownloadRequest req = (HttpDownloadRequest)request;
                HttpCookies cookies = HttpCookies.CreateEmpty();
                cookies.SetCookie(CTOKEN, Ctoken);
                cookies.SetCookie(SESSID, SessionId);
                req.Request.Headers.Add("Cookie", cookies.ToString());
            }

            return AdapterResult.Default;
        }
        #endregion

        
        #region Properties
        public Uri DiscoveryUrl
        {
            get { return DownloadUrl ?? Process.Properties.Url; }
        }

        public string DiscoveryMethod
        {
            get { return WebRequestMethods.Http.Get; }
        }

        public int? MaximumConnections
        {
            get { return 1; }
        }

        public Uri DownloadUrl { get; private set; }

        public bool RequiresRestart { get; private set; }
                
        public IDownloadProcess Process { get; private set; }
        
        internal string SessionId { get; private set; }
        
        internal string Ctoken { get; private set; }
        #endregion
    }
}
