﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.DownloadManager.Network.Protocols.Http;
using System.Web;

namespace WideStream.DownloadManager.Network.Adapters
{
    public sealed class GoogleVideoAdapter : IDownloadAdapter
    {
        #region Construction
        public GoogleVideoAdapter()
        {
        
        }
        #endregion


        #region Methods
        public void Initialize(IDownloadProcess process)
        {
            Process = process;
        }


        public AdapterResult Analyze(IConnectionInformation information, IDownloadResponse response)
        {
            if (VideoUrl == null)
            {
                HttpDownloadResponse httpResponse = (HttpDownloadResponse)response;

                using (StreamReader sr = new StreamReader(httpResponse.Response.GetResponseStream()))
                    ParseProperties(sr.ReadToEnd());
            }

            return AdapterResult.Default;
        }


        public AdapterResult Negotiate(IDownloadRequest request)
        {
            return AdapterResult.Default;
        }


        private void ParseProperties(string source)
        {
            string kw = "videoUrl";
            string chunk = source.Substring(source.IndexOf(kw) + kw.Length);
            int index = chunk.IndexOf("thumbnailUrl");
            VideoUrl = new Uri(HttpUtility.UrlDecode(chunk.Substring(0, index).
                Trim('\\', 'x', '3', 'd', '2', '6' )));
        }
        #endregion


        #region Properties
        public bool RequiresRestart
        {
            get { return true; }
        }


        public Uri DiscoveryUrl
        {
            get { return DownloadUrl; }
        }


        public Uri DownloadUrl
        {
            get { return VideoUrl ?? Process.Properties.Url; }
        }


        internal Uri VideoUrl { get; private set; }
       

        public string DiscoveryMethod
        {
            get
            {
                return VideoUrl == null ? WebRequestMethods.Http.Get :
                    WebRequestMethods.Http.Head;
            }
        }


        public int? MaximumConnections
        {
            get { return 1; }
        }

        public IDownloadProcess Process { get; private set; }
        #endregion
    }
}
