﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.DownloadManager.Network.Protocols.Http;
using System.Web;

namespace WideStream.DownloadManager.Network.Adapters
{
    public sealed class MetacafeAdapter : IDownloadAdapter
    {
        #region Construction
        private const string VAR_URL = "mediaURL";
        private const string VAR_SESSID = "sessionID";
        private const string VAR_GDA = "gdaKey";
        private const string QRY_FORMAT = "{0}?__gda__={1}&aksessionid={2}";

        public MetacafeAdapter()
        {
            Properties = new Dictionary<String,String>();
        }
        #endregion


        #region Methods
        public void Initialize(IDownloadProcess process)
        {
            Process = process;
        }


        public AdapterResult Analyze(IConnectionInformation information, IDownloadResponse response)
        {
            if (VideoUrl == null)
            {
                HttpDownloadResponse httpResponse = (HttpDownloadResponse)response;

                using (StreamReader sr = new StreamReader(httpResponse.Response.GetResponseStream()))
                    ParseProperties(sr.ReadToEnd());
            }

            ((HttpHeaders)information).FileName = VideoName;
            return AdapterResult.Default;
        }


        public AdapterResult Negotiate(IDownloadRequest request)
        {
            return AdapterResult.Default;
        }


        private void ParseProperties(string source)
        {
            string kw = "flashvars";
            string chunk = source.Substring(source.IndexOf(kw) + kw.Length);
            int index = chunk.IndexOf(">");
            string src = HttpUtility.UrlDecode(chunk.Substring(0, index).Trim('=', '&', '"'));

            foreach (string s in src.Split(new char[] { '&' }, StringSplitOptions.RemoveEmptyEntries))
            {
                string[] arr = s.Split('=');
                string key = null;

                if (arr.Length > 1 && !Properties.ContainsKey(key = arr[0].Trim(' ')))
                    Properties.Add(key, arr[1].Trim(' '));
            }
        }
        #endregion


        #region Properties
        public bool RequiresRestart
        {
            get { return true; }
        }


        public Uri DiscoveryUrl
        {
            get { return DownloadUrl; }
        }


        public Uri DownloadUrl
        {
            get { return VideoUrl ?? Process.Properties.Url; }
        }


        private Uri _videoUrl;
        internal Uri VideoUrl 
        {
            get
            {
                if (_videoUrl != null)
                    return _videoUrl;
                if (_videoUrl == null && Properties.Count > 0 && Properties.ContainsKey(VAR_URL))
                {
                    string uri = Properties[VAR_URL];
                    string gda;
                    Properties.TryGetValue(VAR_GDA, out gda);
                    string sessid;
                    Properties.TryGetValue(VAR_SESSID, out sessid);
                    _videoUrl = new Uri(String.Format(QRY_FORMAT, uri, gda, sessid));
                }

                return _videoUrl;
            }
        }


        private string _videoName;
        internal string VideoName 
        {
            get
            {
                if (_videoName == null)
                {
                    string lp = Process.Properties.Url.LocalPath.Trim('/');
                    int index = lp.LastIndexOf('/');
                    string fn = null;

                    if (index > 0 && (fn = lp.Substring(index + 1)).Length > 0)
                        _videoName = fn + ".flv";
                }

                return _videoName;
            }
        }
        

        internal Dictionary<String, String> Properties { get; private set; }
        

        public string DiscoveryMethod
        {
            get
            {
                return VideoUrl == null ? WebRequestMethods.Http.Get :
                    WebRequestMethods.Http.Head;
            }
        }


        public int? MaximumConnections
        {
            get { return 1; }
        }

        public IDownloadProcess Process { get; private set; }
        #endregion
    }
}
