﻿using System;
using System.Net;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.DownloadManager.Network.Protocols.Http;

namespace WideStream.DownloadManager.Network.Adapters
{
    public sealed class RapidshareAdapter : IDownloadAdapter
    {
        #region Construction
        public RapidshareAdapter()
        {

        }
        #endregion


        #region Methods
        public void Initialize(IDownloadProcess process)
        {
            Process = process;
        }


        public AdapterResult Analyze(IConnectionInformation information, IDownloadResponse response)
        {
            return AdapterResult.Default;
        }


        public AdapterResult Negotiate(IDownloadRequest request)
        {
            if (Process.Properties.Credentials != null && 
                !String.IsNullOrEmpty(Process.Properties.Credentials.UserName))
            {
                HttpWebRequest httpRequest = ((HttpDownloadRequest)request).Request;
                HttpCookies cookies = HttpCookies.CreateEmpty();
                cookies.SetCookie("user", String.Format("{0}-{1}",
                    Process.Properties.Credentials.UserName, Process.Properties.Credentials.Password));
                httpRequest.Headers.Add("Cookie", cookies.ToString());
            }

            return AdapterResult.Default;
        }
        #endregion


        #region Properties
        public bool RequiresRestart
        {
            get { return false; }
        }


        public Uri DiscoveryUrl
        {
            get { return Process.Properties.Url; }
        }


        public Uri DownloadUrl
        {
            get { return Process.Properties.Url; }
        }


        public string DiscoveryMethod
        {
            get { return WebRequestMethods.Http.Head; }
        }


        public int? MaximumConnections
        {
            get
            {
                return Process.Properties.Credentials != null &&
                    !String.IsNullOrEmpty(Process.Properties.Credentials.UserName) ? null : (int?)1;
            }
        }

        public IDownloadProcess Process { get; private set; }
        #endregion
    }
}
