﻿using System;
using System.IO;
using System.Net;
using System.Text;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.DownloadManager.Network.Protocols.Http;

namespace WideStream.DownloadManager.Network.Adapters
{
    public sealed class VbulletinAdapter : IDownloadAdapter
    {
        #region Construction
        private const string CONTENT_TYPE = "application/x-www-form-urlencoded";

        public VbulletinAdapter()
        {
            UserAgentKey = Guid.NewGuid().ToString();
        }
        #endregion


        #region Methods
        public void Initialize(IDownloadProcess process)
        {
            Process = process;
        }


        public AdapterResult Analyze(IConnectionInformation information, IDownloadResponse response)
        {
            HttpCookies cookies = ((HttpHeaders)information).Cookies;

            if (cookies != null)
                CookieString = cookies.ToString();
      
            return AdapterResult.Default;
        }


        public AdapterResult Negotiate(IDownloadRequest request)
        {
            HttpDownloadRequest httpRequest = (HttpDownloadRequest)request;
            httpRequest.Request.UserAgent += String.Format("; AdapterHash={0}", UserAgentKey);

            if (String.IsNullOrEmpty(CookieString))
            {
                httpRequest.Form = new HttpForm();
                httpRequest.Form.SetField("do", "login");
                httpRequest.Form.SetField("vb_login_username", Process.Properties.Credentials.UserName);
                httpRequest.Form.SetField("vb_login_password", Process.Properties.Credentials.Password);
            }
            else
                httpRequest.Request.Headers.Add("Cookie", CookieString);
            
            return AdapterResult.Default;
        }


        private Uri ObtainDiscoveryUri()
        {
            if (!String.IsNullOrEmpty(CookieString))
                return Process.Properties.Url;
            else
            {
                string url = Process.Properties.Url.ToString();
                string shortUrl = url.Substring(0, url.LastIndexOf("/")) + "/login.php";
                return new Uri(shortUrl);
            }
        }
        #endregion


        #region Properties
        public bool RequiresRestart 
        {
            get { return HasCredentials; }
        }

        public Uri DiscoveryUrl
        {
            get { return ObtainDiscoveryUri(); }
        }


        public Uri DownloadUrl
        {
            get { return Process.Properties.Url; }
        }


        public string DiscoveryMethod
        {
            get
            {
                return HasCredentials && String.IsNullOrEmpty(CookieString) ? 
                    WebRequestMethods.Http.Post : WebRequestMethods.Http.Head;
            }
        }


        public int? MaximumConnections
        {
            get { return null; }
        }


        internal bool HasCredentials
        {
            get
            {
                return Process.Properties.Credentials != null &&
                    !String.IsNullOrEmpty(Process.Properties.Credentials.UserName);
            }
        }

        public IDownloadProcess Process { get; private set; }

        internal string CookieString { get; private set; }

        internal string UserAgentKey { get; private set; }
        #endregion
    }
}
