﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.DownloadManager.Network.Protocols.Http;

namespace WideStream.DownloadManager.Network.Adapters
{
    public sealed class YoutubeAdapter : IDownloadAdapter
    {
        #region Construction
        public YoutubeAdapter()
        {
            Properties = new Dictionary<String,String>();
        }
        #endregion


        #region Methods
        public void Initialize(IDownloadProcess process)
        {
            Process = process;
        }


        public AdapterResult Analyze(IConnectionInformation information, IDownloadResponse response)
        {
            if (Properties.Count == 0)
            {
                HttpDownloadResponse httpResponse = (HttpDownloadResponse)response;

                using (StreamReader sr = new StreamReader(httpResponse.Response.GetResponseStream()))
                    ParseProperties(sr.ReadToEnd());
            }

            if (Properties.ContainsKey("video_id"))
                ((HttpHeaders)information).FileName = String.Format("youtube{0}.flv", Properties["video_id"]);

            return AdapterResult.Default;
        }


        public AdapterResult Negotiate(IDownloadRequest request)
        {
            return AdapterResult.Default;
        }


        private void ParseProperties(string source)
        {
            string kw = "swfArgs";
            string chunk = source.Substring(source.IndexOf(kw) + kw.Length);
            int index = chunk.IndexOf("}");
            string finalChunk = chunk.Substring(0, index).Trim(' ', '=', '{', '}');
            Properties.Clear();

            foreach (string s in finalChunk.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries))
            {
                string[] arr = s.Split(':');

                if (arr.Length > 1)
                    Properties.Add(arr[0].Trim(' ', '"'), arr[1].Trim(' ', '"'));
            }
        }
        #endregion


        #region Properties
        public bool RequiresRestart
        {
            get { return true; }
        }


        public Uri DiscoveryUrl
        {
            get { return DownloadUrl; }
        }


        public Uri DownloadUrl
        {
            get 
            {
                return Properties.Count > 0 ? new Uri(String.Format("http://youtube.com/get_video?video_id={0}&t={1}",
                    Properties["video_id"], Properties["t"])) : Process.Properties.Url; 
            }
        }


        public string DiscoveryMethod
        {
            get
            {
                return Properties.Count == 0 ? WebRequestMethods.Http.Get :
                    WebRequestMethods.Http.Head;
            }
        }


        public int? MaximumConnections
        {
            get { return 1; }
        }

        internal Dictionary<String,String> Properties { get; private set; }

        public IDownloadProcess Process { get; private set; }
        #endregion
    }
}
