﻿using System;
using System.Linq;
using System.Collections.Generic;
using WideStream.Core.Runtime;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.DownloadManager.Network.Adapters;
using WideStream.Configuration;
using WideStream.DownloadManager.Configuration;
using WideStream.Startup.Resources;

namespace WideStream.DownloadManager.Network
{
    public static class DownloadAdapterManager
    {
        private static Dictionary<String,DownloadAdapterAddin> adapters = new Dictionary<String,DownloadAdapterAddin>();
        
        static DownloadAdapterManager()
        {
            Config.Register<AdapterManagerOptions>();
        }


        
        public static IEnumerable<DownloadAdapterAddin> ObtainAdaptersInformation(Uri url)
        {
            AdapterManagerOptions conf = Config.Query<AdapterManagerOptions>();
            string urlString = url.ToString();

            foreach (var a in adapters.Values.Where(a => !conf.DisabledAdapters.Contains(a.Key) &&
                urlString.Like(a.MatchPattern)))
                yield return a;

            foreach (var a in adapters.Values.Where(a => !conf.DisabledAdapters.Contains(a.Key) &&
                String.IsNullOrEmpty(a.MatchPattern)))
                yield return a;

            yield return GetDefaultAdapter();      
        }


        public static IDownloadAdapter ObtainAdapter(string key)
        {
            return (IDownloadAdapter)Activator.CreateInstance(ObtainAdapterData(key).AdapterType);
        }


        public static DownloadAdapterAddin ObtainAdapterData(string key)
        {
            return String.IsNullOrEmpty(key) || key == "default" ? GetDefaultAdapter() : adapters[key]; 
        }


        public static void RegisterAdapter(DownloadAdapterAddin adapter)
        {
            adapters.Add(adapter.Key, adapter);
        }


        public static IEnumerable<DownloadAdapterAddin> EnumerateAdapters()
        {
            return adapters.Values;
        }


        private static DownloadAdapterAddin GetDefaultAdapter()
        {
            return new DownloadAdapterAddin("default", StringTable.GetString("Adapters.Default"), 
                null, null, typeof(DefaultAdapter), null);
        }
    }
}
