﻿using System;
using WideStream.Core.ComponentModel;
using WideStream.Core.Runtime;
using WideStream.Logging;
using WideStream.Startup.Resources;

namespace WideStream.DownloadManager.Network
{
	public abstract class LogTask : Parallel
	{
		#region Construction
		protected LogTask()
		{

		}
		
		protected LogTask(ILog log)
		{
			Log = log;
		}
		#endregion


		#region Methods
		protected void LogVerbose(string message, params object[] args)
		{
			if (!LogClosed)
				Log.AddEntry(LogEntryType.Verbose, GetMessage(message), args);
		}


		protected void LogInfo(string message, params object[] args)
		{
			if (!LogClosed)
				Log.AddEntry(LogEntryType.Information, GetMessage(message), args);
		}


		protected void LogWarning(string message, params object[] args)
		{
			if (!LogClosed)
				Log.AddEntry(LogEntryType.Warning, GetMessage(message), args);
		}


		protected void LogError(string message, params object[] args)
		{
			if (!LogClosed)
				Log.AddEntry(LogEntryType.Error, GetMessage(message), args);
		}


		protected void LogImportant(string message, params object[] args)
		{
			if (!LogClosed)
				Log.AddEntry(LogEntryType.Important, GetMessage(message), args);
		}


		protected void LogSuccess(string message, params object[] args)
		{
			if (!LogClosed)
                Log.AddEntry(LogEntryType.Success, GetMessage(message), args);
		}


		protected void LogEntry(LogEntry entry)
		{
            if (!LogClosed)
            {
                entry.Message = GetMessage(entry.Message);
                entry.Header = GetMessage(entry.Header);
                Log.AddEntry(entry);
            }
		}


        private string GetMessage(string key)
        {
            return String.IsNullOrEmpty(key) ? key :
                StringTable.GetString(String.Format("Downloads.{0}", key));
        }
		#endregion


		#region Properties
		public ILog Log { get; protected set; }

		protected bool LogClosed { get; set; }
		#endregion
	}
}
