﻿using System;
using System.IO;
using System.Net;
using System.Runtime.CompilerServices;

namespace WideStream.DownloadManager.Network.ObjectModel
{
	public sealed class DownloadFile : IDisposable
	{
		#region Construction
		private DownloadFile(FileInfo file, FileStream stream)
		{
			File = file;
			FileStream = stream;
		}
		#endregion


		#region Methods
		public static DownloadFile Create(FileInfo file, long? length)
		{
			if (file.Exists)
				return null;
			else
			{
				FileStream stream = System.IO.File.Create(file.FullName);
				
				if (length != null)
					stream.SetLength(length.Value);

				return new DownloadFile(file, stream);
			}			
		}


		public static DownloadFile Open(FileInfo file)
		{
			if (!file.Exists)
				return null;
			else
			{
				FileStream stream = System.IO.File.Open(file.FullName, FileMode.OpenOrCreate, FileAccess.ReadWrite);
				stream.Seek(stream.Length, SeekOrigin.Begin);
				return new DownloadFile(file, stream);
			}
		}


		[MethodImpl(MethodImplOptions.Synchronized)]
		public void Write(byte[] buffer, int length, long offset)
		{
            Stream fs = FileStream;

            if (fs != null && fs.CanWrite)
            {
                fs.Seek(offset, SeekOrigin.Begin);
                fs.Write(buffer, 0, length);
            }
		}


		public void Dispose()
		{
			if (FileStream != null)
				lock (this)
					if (FileStream != null)
					{
						FileStream.Dispose();
						FileStream = null;
					}
		}
		#endregion


		#region Properties
		public FileInfo File { get; private set; }

		public FileStream FileStream { get; private set; }
		#endregion
	}
}
