﻿using System;
using System.Collections.Generic;
using WideStream.Configuration;
using WideStream.Configuration.ComponentModel;
using WideStream.Core.Network;
using WideStream.DownloadManager.Configuration;

namespace WideStream.DownloadManager.Network.ObjectModel
{
    public sealed class DownloadProperties : Options
    {
        #region Construction
        public DownloadProperties()
        {
			ProtocolOptions = new List<Options>();
			GeneralOptions = new DownloadProcessOptions();
			State = new DownloadState();

			foreach (DownloadProtocol t in DownloadProtocolManager.EnumerateDownloadTypes())
				ProtocolOptions.Add(t.Options.Clone());
        }
        #endregion


		#region Methods
		public new DownloadProperties Clone()
		{
			return (DownloadProperties)MemberwiseClone();
		}


		public void Refresh()
		{
			OnUrlChanged();
			GeneralOptions.OnPriorityChanged();
		}

		public string GetDisplayName()
		{
			if (Config.Query<DownloadManagerDisplayOptions>().ShowFileNameOnly)
			{
				if (State != null && State.FileLocation != null)
					return State.FileLocation.Name;
				else
				{
					string loc = Url.LocalPath;
					int index;

					if ((index = loc.LastIndexOf("/")) != -1)
						loc = loc.Substring(index + 1);

					return loc;
				}
			}
			else
				return Url.ToString();
		}
		#endregion


		#region Events
		public event EventHandler UrlChanged;
        private void OnUrlChanged()
        {
            EventHandler h = UrlChanged;

            if (h != null)
                h(this, EventArgs.Empty);
        }
        #endregion


        #region Properties
		private Uri _url;
		public Uri Url
		{
			get { return _url; }
			set
			{
				if (_url != value)
				{
					_url = value;
					OnUrlChanged();
				}
			}
		}

        [SaveRule(Ignore = true)]
        public Uri RedirectUrl { get; set; }

        public string Adapter { get; set; }

		public string NewName { get; set; }

		public Credentials Credentials { get; set; }

		[SaveRule(Rule = SaveRule.NestedOptions)]
		public DownloadProcessOptions GeneralOptions { get; set; }

		[SaveRule(Rule = SaveRule.Collection)]
		public List<Options> ProtocolOptions { get; private set; }

		[SaveRule(Rule = SaveRule.NestedOptions)]
		public DownloadState State { get; set; }
        #endregion
    }
}
