﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Runtime.CompilerServices;
using WideStream.Configuration.ComponentModel;

namespace WideStream.DownloadManager.Network.ObjectModel
{
	public sealed class DownloadState : IDisposable
	{
		#region Construction
		public DownloadState()
		{
			Segments = new List<Segment>();
		}

		~DownloadState()
		{
			Close();
		}
		#endregion


		#region Methods
		public void Close()
		{
			Close(false, false);
		}


		[MethodImpl(MethodImplOptions.Synchronized)]
		public void Close(bool nonRecovery, bool removeFile)
		{	
			ErrorCount = 0;
			((IDisposable)this).Dispose();

			if (nonRecovery)
			{
				ResumeMode = false;
				StartTime = default(DateTime);
				Segments.Clear();
			}
			
			if (removeFile && File != null && File.File != null)
				File.File.Delete();

			File = null;
		}


		[MethodImpl(MethodImplOptions.Synchronized)]
		void IDisposable.Dispose()
		{
			try { File.Dispose(); }
			catch { }
			GC.SuppressFinalize(this);
		}
		#endregion


		#region Properties
		internal long BytesRead
		{
			get { return Segments.Sum(s => s.Read); }
		}

		internal long ContentLength
		{
			get { return Segments.Sum(s => s.Length); }
		}
		
		public DownloadStatus PreviousStatus { get; set; }
		
		internal int ErrorCount { get; set; }

		public bool ResumeMode { get; set; }

		internal DateTime StartTime { get; set; }

		public FileInfo FileLocation { get; set; }

		internal DownloadFile File { get; set; }

		[SaveRule(Rule = SaveRule.Collection, ChildType = typeof(Segment))]
		public List<Segment> Segments { get; private set; }
		#endregion
	}
}