﻿using System;
using System.Linq;
using System.Net;
using System.Reflection;
using WideStream.DownloadManager.Network;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.Core.Network;

namespace WideStream.DownloadManager.Network.Protocols.Ftp
{
	public sealed class FtpDownloadConfiguration : DownloadConfiguration
	{
		#region Construction
		public FtpDownloadConfiguration(IDownloadProcess process) : base(process)
		{

		}
		#endregion


		#region Methods
		public void FullConfigure(IDownloadRequest request)
		{
			Configure(request);
			Authorize(request);
			Negotiate(request);
		}


		public override void Configure(IDownloadRequest request)
		{
			base.Configure(request);
			FtpOptions options = (FtpOptions)Properties.ProtocolOptions.First(o => o is FtpOptions);
			FtpWebRequest ftpRequest = ((FtpDownloadRequest)request).Request;
			ftpRequest.Timeout = (Int32)Properties.GeneralOptions.Timeout;
			ftpRequest.UsePassive = options.UsePassiveMode;
			ftpRequest.KeepAlive = options.KeepAlive;
			ftpRequest.ReadWriteTimeout = (Int32)Properties.GeneralOptions.Timeout;
			ftpRequest.EnableSsl = options.EnableSsl;
		}


		public override void Authorize(IDownloadRequest request)
		{
			if (Properties.Credentials != null)
			{
				FtpWebRequest ftpRequest = ((FtpDownloadRequest)request).Request;

				if (Properties.Credentials != null)
					ftpRequest.Credentials = Properties.Credentials;
			}
			else
				Properties.Credentials = new Credentials("anonymous", "user@widestream.org");
		}


		public override void Negotiate(IDownloadRequest request)
		{
			FtpWebRequest ftpRequest = ((FtpDownloadRequest)request).Request;
			ftpRequest.ServicePoint.ConnectionLimit = Process.MaximumConnections;
		}


		public override void PrepareForDownload(IDownloadRequest request, Segment segment)
		{
			if (Process.ConnectionInformation != null &&
				Process.ConnectionInformation.SupportRanges)
			{
				FtpWebRequest ftpRequest = ((FtpDownloadRequest)request).Request;
				ftpRequest.ContentOffset = segment.Offset + segment.Read;
				
				if (segment.Read > 0)
                    LogInfo("SegmentResumeFrom", segment.Id, segment.Offset + segment.Read); 
			}
		}
		#endregion
	}
}
