﻿using System;
using System.IO;
using System.Net;
using WideStream.DownloadManager.Network.ObjectModel;

namespace WideStream.DownloadManager.Network.Protocols.Ftp
{
	public sealed class FtpDownloadConnection : DownloadConnection, IDownloadConnection
	{
		#region Construction
		public FtpDownloadConnection(IDownloadProcess process) : base(process)
		{
			
		}
		#endregion


		#region Methods
		protected override Stream ObtainResponseStream(IDownloadResponse response)
		{
			return ((FtpDownloadResponse)response).Response.GetResponseStream();
		}


		protected override void AnalyzeResponse(IDownloadResponse response)
		{
			FtpDownloadResponse ftpResponse = (FtpDownloadResponse)response;
			Response = ftpResponse.Response;
			base.AnalyzeResponse(response);
		}


		protected override bool AnalyzeReadResult(int read)
		{
			if (Response.StatusCode != LastStatusCode)
			{
				LogImportant(Response.StatusDescription);
				LastStatusCode = Response.StatusCode;
			}

			return base.AnalyzeReadResult(read);
		}
		#endregion


		#region Properties
		internal new FtpWebResponse Response { get; set; }

		public FtpStatusCode LastStatusCode { get; private set; }
		#endregion
	}
}
