﻿using System;
using System.IO;
using System.Net;
using System.Text;

namespace WideStream.DownloadManager.Network.Protocols.Ftp
{
	public sealed class FtpFeatures : ConnectionInformation
	{
		#region Construction
		private FtpFeatures()
		{

		}
		#endregion


		#region Methods
		public void UpdateContentData(FtpWebResponse response)
		{
			ContentLength = response.ContentLength;
			FileDate = response.LastModified;
		}


		public static FtpFeatures ParseFeatures(string features)
		{
			FtpFeatures ret = AnalyzeFeat(features);
			ret.ResponseString = features;
			return ret;
		}


		public static FtpFeatures CreateDefault()
		{
			return new FtpFeatures();
		}


		private static FtpFeatures AnalyzeFeat(string response)
		{
			FtpFeatures feat = new FtpFeatures();
			StringReader sr = new StringReader(response);
			string line;

			while ((line = sr.ReadLine()) != null)
			{
				if (line.Contains("REST"))
					feat.SupportRanges = true;
				
				if (line.Contains("SIZE"))
					feat.SupportFileSizeRequest = true;
			}

			return feat;
		}		
		#endregion


		#region Properties
		public bool SupportFileSizeRequest { get; private set; }
		#endregion
	}
}
