﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using WideStream.DownloadManager.Network.ObjectModel;
using System.Collections;
using WideStream.Core.Runtime;
using System.Text;

namespace WideStream.DownloadManager.Network.Protocols.Http
{
    public sealed class HttpCookies : IWebCookies
    {
        #region Construction
        private Dictionary<String,String> cookies;

        private HttpCookies()
        {
            cookies = new Dictionary<String,String>();
        }
        #endregion


        #region Methods
        public static HttpCookies Parse(string cookies)
        {
            string[] arr = cookies.Split(new char[] { ';' }, StringSplitOptions.RemoveEmptyEntries);
            HttpCookies ret = new HttpCookies();

            foreach (string s in arr)
                ParseElement(ret.cookies, s);

            return ret;
        }


        public static HttpCookies CreateEmpty()
        {
            return new HttpCookies();
        }


        private static void ParseElement(Dictionary<String,String> dict, string element)
        {
            string[] arr = element.Split(new char[] { '=' }, StringSplitOptions.RemoveEmptyEntries);

            if (arr.Length > 2)
            {
                string[] carr = element.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);

                if (carr.Length > 1)
                {
                    foreach (string s in carr)
                        ParseElement(dict, s);
                }
            }

            if (arr.Length == 2)
            {
                string key = arr[0].Trim();

                if (!dict.ContainsKey(key))
                    dict.Add(key, arr[1].Trim());
            }
        }


        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();

            foreach (string k in cookies.Keys)
            {
                if (sb.Length > 0)
                    sb.Append("; ");

                sb.AppendFormat("{0}={1}", k, cookies[k]);
            }

            return sb.ToString();
        }


        public string GetCookie(string key)
        {
            if (cookies.ContainsKey(key))
                return cookies[key];
            else
                return String.Empty;
        }


        public void SetCookie(string key, string value)
        {
            if (cookies.ContainsKey(key))
                cookies[key] = value;
            else
                cookies.Add(key, value);
        }
      

        public IEnumerator<ValuePair<String,String>> GetEnumerator()
        {
            foreach (string k in cookies.Keys)
                yield return new ValuePair<String,String>(k, cookies[k]);
        }
               

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }
        #endregion


        #region Properties
        public int Count
        {
            get { return cookies.Count; }
        }
        #endregion
    }
}
