﻿using System;
using System.Linq;
using System.Net;
using System.Reflection;
using WideStream.DownloadManager.Network;
using WideStream.DownloadManager.Network.ObjectModel;

namespace WideStream.DownloadManager.Network.Protocols.Http
{
	public sealed class HttpDownloadConfiguration : DownloadConfiguration
	{
		#region Construction
		public HttpDownloadConfiguration(IDownloadProcess process) : base(process)
		{

		}
		#endregion


		#region Methods
		public void ConfigureAll(IDownloadRequest request)
		{
			Configure(request);
			Authorize(request);
			Negotiate(request);
		}


		public override void Configure(IDownloadRequest request)
		{
			base.Configure(request);
			
			HttpWebRequest httpRequest = ((HttpDownloadRequest)request).Request;
			HttpOptions options = (HttpOptions)Process.Properties.ProtocolOptions.First(o => o is HttpOptions);
			httpRequest.Timeout = (Int32)Process.Properties.GeneralOptions.Timeout;
			httpRequest.ReadWriteTimeout = (Int32)Properties.GeneralOptions.Timeout;
			options.AllowAutoRedirect = options.AllowAutoRedirect;

			if (options.MaximumRedirections > 0)
				httpRequest.MaximumAutomaticRedirections = (Int32)options.MaximumRedirections;

			httpRequest.ProtocolVersion = options.ProtocolVersion == HttpProtocolVersion.Http11 ? 
				HttpVersion.Version11 : HttpVersion.Version10;
			httpRequest.KeepAlive = options.KeepAlive;
			
			httpRequest.UserAgent = String.Format("Mozilla/4.0 (compatible; WideStream; {0}; CLR {1})",
				Environment.OSVersion.VersionString, Environment.Version.ToString(3));
            httpRequest.Referer = Process.Adapter.DiscoveryUrl.ToString();
			httpRequest.Accept = "*/*";
		}


		public override void Authorize(IDownloadRequest request)
		{
            if (Properties.Credentials != null && !String.IsNullOrEmpty(Properties.Credentials.UserName))
			{
				HttpWebRequest httpRequest = ((HttpDownloadRequest)request).Request;				
				httpRequest.Credentials = Properties.Credentials;
          	}
		}


		public override void Negotiate(IDownloadRequest request)
		{
			HttpWebRequest httpRequest = ((HttpDownloadRequest)request).Request;
            httpRequest.ServicePoint.ConnectionLimit = (Int32)Properties.GeneralOptions.
                MaximumDownloadConnections * 100;
		}


		public override void PrepareForDownload(IDownloadRequest request, Segment segment)
		{
            if (Process.ConnectionInformation != null && Process.ConnectionInformation.SupportRanges && 
                segment.Offset + segment.Read > 0)
            {
                SetRange(request, segment.Offset + segment.Read, segment.Offset + segment.Length);
             
                if (segment.Read > 0)
                    LogInfo("SegmentResumeFrom", segment.Id, segment.Offset + segment.Read);
            }
		}


		private void SetRange(IDownloadRequest request, long offset, long length)
		{
			HttpWebRequest httpRequest = ((HttpDownloadRequest)request).Request;
			MethodInfo mi = typeof(HttpWebRequest).GetMethod("AddRange", 
				BindingFlags.Instance | BindingFlags.NonPublic);
			mi.Invoke(httpRequest, new object[] { "bytes", offset.ToString(), length.ToString() });
		}
		#endregion
	}
}
