﻿using System;
using System.Net;
using System.Text;
using System.IO;
using WideStream.DownloadManager.Network.ObjectModel;

namespace WideStream.DownloadManager.Network.Protocols.Http
{
	public sealed class HttpHeaders : ConnectionInformation
	{
		#region Construction
		internal HttpHeaders()
		{
			
		}
		#endregion


		#region Methods
		public static HttpHeaders Read(WebHeaderCollection headers)
		{
			StringBuilder sb = new StringBuilder();
			HttpHeaders ret = new HttpHeaders();

			foreach (string s in headers.Keys)
			{
				if (String.Compare(s, "Accept-Ranges", true) == 0)
					ret.SupportRanges = true;

				if (String.Compare(s, "Transfer-Encoding", true) == 0 &&
					String.Compare(headers[s], "chunked", true) == 0)
					ret.ChunkedMode = true;

				if (String.Compare(s, "Set-Cookie", true) == 0)
					ret.Cookies = HttpCookies.Parse(headers[s]);
				
				if (String.Compare(s, "Content-Length", true) == 0)
					ret.ContentLength = Int64.Parse(headers[s]);

				if (String.Compare(s, "Content-Type", true) == 0)
					ret.ContentType = headers[s];

                if (String.Compare(s, "Location", true) == 0)
                {
                    ret.Location = headers[s];
                    ret.NeedRestart = true;
                }

				if (String.Compare(s, "Date", true) == 0)
				{
					DateTime dt;
					if (!DateTime.TryParse(headers[s], out dt))
						ret.FileDate = DateTime.Now;
					else
						ret.FileDate = dt;
				}

				if (String.Compare(s, "Content-Disposition", true) == 0)
					ret.FileName = ParseFileName(headers[s]);

				sb.AppendLine(String.Format("{0}: {1}", s, headers[s]));
			}

			if (ret.ChunkedMode)
				ret.ContentLength = Int64.MaxValue;
			else if (ret.ContentLength <= 0)
			{
				ret.ContentLength = Int64.MaxValue;
				ret.ChunkedMode = true;
			}

			ret.ResponseString = sb.ToString();			
			return ret;
		}


		private static string ParseFileName(string value)
		{
			if (value != null)
			{
				try
				{
					int index = value.IndexOf("filename=");

					if (index != -1)
					{
						value = value.Substring(index + 9).Trim(' ', '\"');

						if ((index = value.IndexOf("\"")) != -1)
							value = value.Substring(0, index);

						return value;
					}
				}
                catch (ArgumentOutOfRangeException)
				{
					return null;
				}
			}

			return null;
		}
		#endregion


        #region Properties
        public HttpCookies Cookies { get; private set; }
        #endregion
    }
}
