﻿using System;
using System.Text.RegularExpressions;
using System.Collections.Generic;

namespace WideStream.DownloadManager.Network
{
    public sealed class UrlSpider
    {
        #region Construction
        private const string REGEX = "(?:[hH][rR][eE][fF]\\s*=\\s*[\'\\\"])(?:[\\s]*)(.*?)(?:[\'\\\"])";

        public UrlSpider(params string[] protocols)
        {
            AllowedProtocols = new List<String>();

            if (protocols.Length > 0)
                AllowedProtocols.AddRange(protocols);
        }
        #endregion


        #region Methods
        public IEnumerable<Uri> Parse(Uri baseUrl, string source)
        {
            MatchCollection matches = Regex.Matches(source, REGEX, 
                RegexOptions.Compiled | RegexOptions.IgnoreCase | RegexOptions.Singleline);

            if (matches.Count != 0)
            {
                foreach (Match m in matches)
                {
                    Uri uri;

                    if (Uri.TryCreate(m.Groups[1].Value, UriKind.RelativeOrAbsolute, out uri))
                    {
                        if (!uri.IsAbsoluteUri)
                            uri = new Uri(baseUrl, uri);
                        
                        if (AllowedProtocols.Count == 0 ||
                            AllowedProtocols.IndexOf(uri.Scheme) != -1)
                            yield return uri;
                    }
                }
            }
        }
        #endregion


        #region Properties
        public List<String> AllowedProtocols { get; private set; }
        #endregion
    }
}
