﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using WideStream.Configuration;
using WideStream.Core.Runtime;
using WideStream.DownloadManager.Configuration;

namespace WideStream.DownloadManager.Presentation
{
	public static class ActiveDownloadsManager
	{
		private static ActiveDownloadsWindow window = new ActiveDownloadsWindow();
		private static List<DownloadUserControl> controls = new List<DownloadUserControl>();

		static ActiveDownloadsManager()
		{
			Config.ConfigurationChanged += (o, e) => ApplyChanges();
			window.Opacity = .0D;
			window.Show();
			window.Hide();
		}


		public static void ShowActiveDownloadsWindow()
		{
			AdjustHeight();
			ApplyChanges();
			ActiveDownloadsOptions conf = Config.Query<ActiveDownloadsOptions>();
			window.SafeExecute(() => window.ShowWindow());
			
		}

		public static void HideActiveDownloadsWindow()
		{
			if (WindowShown)
				window.SafeExecute(() => window.HideAndRememberPosition());
		}


		public static void AddActiveDownload(DownloadUserControl control)
		{
			if (!window.Visible && Config.Query<ActiveDownloadsOptions>().AutoShowHide)
				ShowActiveDownloadsWindow();
			
			AddControl(control);
		}


		private static void AddControl(DownloadUserControl control)
		{
			if (window.IsHandleCreated && !window.IsDisposed)
				window.SafeExecute(() =>
				{
					control.Dock = DockStyle.Bottom;
					window.Controls.Add(control);
					AdjustHeight();
				});
		}


		public static void RemoveActiveDownload(DownloadUserControl control)
		{
			if (window.IsHandleCreated && !window.IsDisposed)
			{
				window.SafeExecute(() =>
				{
					window.Controls.Remove(control);
					AdjustHeight();
				});

				if (window.Controls.Count == 1 && Config.Query<ActiveDownloadsOptions>().AutoShowHide)
					HideActiveDownloadsWindow();
			}
		}


		private static void AdjustHeight()
		{
			if (window.Controls.Count < 2)
				window.Height = 50;
			else
				window.Height = (window.Controls.Count - 1) * window.Controls[1].Height + 21;
		}


		private static void ApplyChanges()
		{
			window.SafeExecute(() => {
				ActiveDownloadsOptions conf = Config.Query<ActiveDownloadsOptions>();
				window.Opacity = conf.Transparent ? .8D : 1.0D;
				window.ShowInTaskbar = !(window.TopMost = conf.TopMost);
				window.Opacity = conf.Transparent ? .8D : 1.0D;
				window.StartPosition = conf.WindowPositionSave == WindowPositionSaveOption.AlwaysCentered ?
					FormStartPosition.CenterScreen : conf.WindowPositionSave == WindowPositionSaveOption.WindowsControlled ?
					FormStartPosition.WindowsDefaultLocation : FormStartPosition.Manual;

				if (conf.WindowPositionSave == WindowPositionSaveOption.Save &&
					conf.Top != 0 && conf.Left != 0)
				{
					window.Top = conf.Top;
					window.Left = conf.Left;
				}

				window.Refresh();
			});
		}


		public static bool WindowShown
		{
			get { return window.Visible; }
		}
	}
}
