﻿using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using WideStream.Configuration;
using WideStream.DownloadManager.Configuration;
using WideStream.Startup.Resources;

namespace WideStream.DownloadManager.Presentation
{
	public partial class ActiveDownloadsWindow : Form
	{
		public const int WM_NCLBUTTONDOWN = 0xA1;
		public const int HT_CAPTION = 0x2;

		[DllImportAttribute("user32.dll")]
		public static extern int SendMessage(IntPtr hWnd, int Msg, int wParam, int lParam);

		[DllImportAttribute("user32.dll")]
		public static extern bool ReleaseCapture();

        private Timer timer;


		public ActiveDownloadsWindow()
		{
            InitializeComponent();
		}


        public void HideAndRememberPosition()
        {
            Config.Query<ActiveDownloadsOptions>().Top = Top;
			Config.Query<ActiveDownloadsOptions>().Left = Left;
            timer.Stop();
            timer.Dispose();
            timer = null;
            base.Hide();
        }


        public void ShowWindow()
        {
            base.Show();
            timer = new Timer();
            timer.Interval = 1000;
            timer.Tick += (o, ev) =>
            {
                foreach (Control c in Controls)
                {
                    DownloadUserControl uc;

                    if ((uc = c as DownloadUserControl) != null)
                        uc.UpdateData();
                }
            };
            timer.Start();
        }


		protected override void OnMouseDown(MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left)
			{
				ReleaseCapture();
				SendMessage(Handle, WM_NCLBUTTONDOWN, HT_CAPTION, 0);
			}

			base.OnMouseDown(e);
		}


		protected override void OnPaint(PaintEventArgs e)
		{
			ActiveDownloadsOptions opt = Config.Query<ActiveDownloadsOptions>();			
			
			using (Brush b = new SolidBrush(opt.HeaderBackColor))
				e.Graphics.FillRectangle(b, new Rectangle(0, 0, ClientSize.Width, 20));

			e.Graphics.DrawLine(SystemPens.ControlDark, new Point(0, 20), new Point(ClientRectangle.Width, 20));

			using (Brush b = new SolidBrush(opt.HeaderTextColor))
				e.Graphics.DrawString(StringTable.GetString("ActiveDownloads.DialogTitle"), 
                    new Font(Font, FontStyle.Bold), b, new PointF(5, 4));

			if (Controls.Count == 1)
                e.Graphics.DrawString(StringTable.GetString("ActiveDownloads.NoDownloads"), 
                    new Font(Font, FontStyle.Italic), SystemBrushes.ControlText, new PointF(5, 27));

			e.Graphics.DrawRectangle(SystemPens.ControlDark, new Rectangle(
				ClientRectangle.X, ClientRectangle.Y, ClientRectangle.Width - 1, ClientRectangle.Height - 1));
			base.OnPaint(e);
		}


        private void closeButton_Click(object sender, EventArgs e)
		{
            HideAndRememberPosition();
		}

		private void closeButton_Paint(object sender, PaintEventArgs e)
		{
			ActiveDownloadsOptions opt = Config.Query<ActiveDownloadsOptions>();
			
			using (Brush b = new SolidBrush(opt.HeaderBackColor))
				e.Graphics.FillRectangle(b, ClientRectangle);

			using (Pen p = new Pen(opt.HeaderTextColor))
			{
				e.Graphics.DrawLine(p, 1, 0, 8, 7);
				e.Graphics.DrawLine(p, 2, 0, 9, 7);
				e.Graphics.DrawLine(p, 8, 0, 1, 7);
				e.Graphics.DrawLine(p, 9, 0, 2, 7);
			}
		}
	}
}
