﻿using System;
using System.Drawing;
using System.Windows.Forms;
using WideStream.Configuration;
using WideStream.Core.Network;
using WideStream.DownloadManager.Configuration;
using WideStream.Startup.Resources;

namespace WideStream.DownloadManager.Presentation
{
	public partial class DownloadUserControl : UserControl
	{
		public DownloadUserControl()
		{
			InitializeComponent();
		}


		public void UpdateData()
		{
            this.Invoke((MethodInvoker)(() =>
            {
                downloadLabel.Text = DownloadName;
                progressBar.Value = Progress;
                speedLabel.Text = String.Format(StringTable.GetString("DownloadPopup.Connections"),
                    Downloaded, DownloadSize, Speed.ToString(SizeFormat.Speed), Connections,
                    TimeRemaining != default(TimeSpan) && TimeRemaining.TotalMilliseconds > 0 ?
                        String.Format(StringTable.GetString("DownloadPopup.Left"), TimeRemaining) : String.Empty);
            }));
		}

        public string DownloadName { get; set; }

        public int Progress { get; set; }

        public TimeSpan TimeRemaining { get; set; }

		public DataSize Speed { get; set; }

		public DataSize Downloaded { get; set; }

		public DataSize DownloadSize { get; set; }

		public int Connections { get; set; }

		private void DownloadUserControl_Resize(object sender, EventArgs e)
		{
			downloadLabel.Width = ClientSize.Width - downloadLabel.Left * 2;
			speedLabel.Width = ClientSize.Width - speedLabel.Left * 2;
			progressBar.Width = ClientSize.Width - progressBar.Left * 2;
		}


		protected override void OnPaint(PaintEventArgs e)
		{
			ActiveDownloadsOptions opt = Config.Query<ActiveDownloadsOptions>();
            Control parent = Parent;

            if (parent != null)
            {
                BackColor = parent.Controls.IndexOf(this) % 2 == 0 ? opt.BackColor :
                    opt.AltBackColor;
                e.Graphics.DrawLine(SystemPens.ControlDark, new Point(0, ClientSize.Height - 1),
                    new Point(ClientSize.Width, ClientSize.Height - 1));
            }

			base.OnPaint(e);
		}

        private void DownloadUserControl_Load(object sender, EventArgs e)
        {
            downloadLabel.Text = StringTable.GetString(downloadLabel.Text);
        }
	}
}
