﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.DownloadManager.Network;
using WideStream.Core.Network;
using System.IO;
using WideStream.HostManager.Configuration;
using WideStream.Workspace.Forms;
using WideStream.Startup.Resources;

namespace WideStream.DownloadManager.Presentation
{
	public partial class GeneralPropertiesControl : UserControl
	{
		#region Construction
		public GeneralPropertiesControl()
		{
			InitializeComponent();
			UrlAnalyzer = new UrlAnalyzer(false);
		}
		#endregion


		#region Methods
		public bool IsValid()
		{
			foreach (Control c in panel.Controls)
			    errorProvider.SetError(c, null);
			
			if (credentialsCheckBox.Checked && String.IsNullOrEmpty(userNameTextBox.Text))
				return ReturnError(userNameTextBox, "User name is required!");
			else if (credentialsCheckBox.Checked && String.IsNullOrEmpty(passwordTextBox.Text))
				return ReturnError(passwordTextBox, "Password is required!");
			else if (!ValidateTextField(folderTextBox))
				return false;
			else if (!ValidateTextField(linkTextBox))
				return false;
			else if (!ValidateTextField(renameTextBox))
				return false;

			return true;
		}
		

		private void credentialsCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			userNameTextBox.Enabled = passwordTextBox.Enabled = credentialsCheckBox.Checked;
		}


		private void credentials_TextChanged(object sender, EventArgs e)
		{
			Uri uri;

			if (Uri.TryCreate(linkTextBox.Text, UriKind.Absolute, out uri))
			{
				if (!String.IsNullOrEmpty(uri.UserInfo))
				{
					DisableAutoUpdate = true;

					if (!String.IsNullOrEmpty(passwordTextBox.Text) &&
						!String.IsNullOrEmpty(userNameTextBox.Text))
						linkTextBox.Text = linkTextBox.Text.Replace(uri.UserInfo,
							String.Format("{0}:{1}", userNameTextBox.Text, passwordTextBox.Text));
					else if (!String.IsNullOrEmpty(userNameTextBox.Text))
						linkTextBox.Text = linkTextBox.Text.Replace(uri.UserInfo, userNameTextBox.Text);

					DisableAutoUpdate = false;
				}
			}
		}


		private void emptyRenameLabel_Click(object sender, EventArgs e)
		{
			emptyRenameLabel.Visible = false;
			renameTextBox.Focus();
		}
		

		private void renameTextBox_Leave(object sender, EventArgs e)
		{
			emptyRenameLabel.Visible = renameTextBox.Text.Length == 0;
		}


		private void renameTextBox_Enter(object sender, EventArgs e)
		{
			emptyRenameLabel.Visible = false;
		}


		private void renameTextBox_TextChanged(object sender, EventArgs e)
		{
			emptyRenameLabel.Visible = renameTextBox.Text.Length == 0;
		}


		private void linkTextBox_TextChanged(object sender, EventArgs e)
		{
			if (DisableAutoUpdate)
                return;

			int selLength = linkTextBox.SelectionLength;
			int selStart = linkTextBox.SelectionStart;
            DisableAutoUpdate = true;
			AnalysisResult res = UrlAnalyzer.AnalyzeInput(new CurrentData { RenameValue = renameTextBox.Text, 
                HasCredentials = (userNameTextBox.Text.Length > 0), InputValue = linkTextBox.Text });
        
			if ((res.Actions & RequiredActions.ClearCredentials) == RequiredActions.ClearCredentials)
			{
				credentialsCheckBox.Checked = false;
				userNameTextBox.Text = passwordTextBox.Text = String.Empty;
			}

			if ((res.Actions & RequiredActions.SetCredentials) == RequiredActions.SetCredentials)
			{
				credentialsCheckBox.Checked = true;
				userNameTextBox.Text = res.Credentials.UserName;
				passwordTextBox.Text = res.Credentials.Password;
			}

			if ((res.Actions & RequiredActions.UpdateInput) == RequiredActions.UpdateInput)
			{
				linkTextBox.Text = res.InputValue;
				UrlAnalyzer.ChangeState(linkTextBox.Text, null);
			}

			if ((res.Actions & RequiredActions.UpdateLink) == RequiredActions.UpdateLink)
			{
				OnUpdateLink();				
			}

			if ((res.Actions & RequiredActions.AddEmptyDownload) == RequiredActions.AddEmptyDownload)
			{
				linkTextBox.Text = res.InputValue;
				OnAddEmptyDownload();
			}

			if ((res.Actions & RequiredActions.UpdateLinkList) == RequiredActions.UpdateLinkList)
			{
				OnUpdateLinkList(new UpdateLinksEventArgs(res.Downloads));

				if (res.InputValue != null)
					linkTextBox.Text = res.InputValue;
			}

			if ((res.Actions & RequiredActions.SetHostDefaultData) == RequiredActions.SetHostDefaultData)
			{
				OnHostRecognized(new HostEventArgs(res.SampleHost));
			}

			CheckCurrentAdapter();
			linkTextBox.SelectionLength = selLength;
			linkTextBox.SelectionStart = selStart;
			DisableAutoUpdate = false;
		}


		private void folderButton_Click(object sender, EventArgs e)
		{
			if (folderDialog.ShowDialog(this) == DialogResult.OK)
				folderTextBox.Text = folderDialog.SelectedPath;
		}


		private void CheckCurrentAdapter()
		{
			Uri url;

			if (Uri.TryCreate(linkTextBox.Text, UriKind.Absolute, out url) &&
				DownloadProtocolManager.IdentifyProtocol(url) == null)
				errorProvider.SetError(linkTextBox, "Unknown protocol!");
			else
				errorProvider.SetError(linkTextBox, null);

            adapterCombo.Items.Clear();

            if (url != null)
            {
                adapterCombo.DefaultImage = Image.FromStream(typeof(GeneralPropertiesControl).Assembly.
                    GetManifestResourceStream("WideStream.Images.web.bmp"));

                foreach (var a in DownloadAdapterManager.ObtainAdaptersInformation(url))
                    adapterCombo.Items.Add(new IconListEntry { Icon = a.Image, Text = a.Name, Value = a.Key });

                if (adapterCombo.Items.Count > 0)
                    adapterCombo.SelectedIndex = 0;
            }                
		}	


		private bool ValidateTextField(TextBox tb)
		{
			if (tb.Name == folderTextBox.Name)
			{
				DirectoryInfo dir = new DirectoryInfo(tb.Text);

				if (!dir.Exists)
					return ReturnError(tb, "A path to the folder is not correct!");
			}
			else if (tb.Name == linkTextBox.Name)
			{
				Uri uri;

				if (!Uri.TryCreate(tb.Text, UriKind.Absolute, out uri))
					return ReturnError(tb, "An URI is invalid!");
			}
			else if (tb.Name == renameTextBox.Name)
			{
				try
				{
					FileInfo fi = null;

					if (!String.IsNullOrEmpty(tb.Text))
						fi = new FileInfo(tb.Text);
				}
				catch (ArgumentException)
				{
					return ReturnError(tb, "File name contains invalid symbols!");
				}
				catch (PathTooLongException)
				{
					return ReturnError(tb, "The selected path is tool long!");
				}
				catch (NotSupportedException)
				{
					return ReturnError(tb, "File name can't contain a semicolon symbol!");
				}
			}

			return true;
		}


		private bool ReturnError(Control control, string error)
		{
			errorProvider.SetError(control, error);
			return false;
		}


		public void FocusLinkTextBox()
		{
			linkTextBox.Focus();
		}
		#endregion


		#region Events
		public event EventHandler BuildLinkList;
		private void OnBuildLinkList()
		{
			EventHandler h = BuildLinkList;

			if (h != null)
				h(this, EventArgs.Empty);
		}


		public event EventHandler<UpdateLinksEventArgs> UpdateLinkList;
		private void OnUpdateLinkList(UpdateLinksEventArgs e)
		{
			EventHandler<UpdateLinksEventArgs> h = UpdateLinkList;

			if (h != null)
				h(this, e);
		}


		public event EventHandler AddEmptyDownload;
		private void OnAddEmptyDownload()
		{
			EventHandler h = AddEmptyDownload;

			if (h != null)
				h(this, EventArgs.Empty);
		}


		public event EventHandler UpdateLink;
		private void OnUpdateLink()
		{
			EventHandler h = UpdateLink;

			if (h != null)
				h(this, EventArgs.Empty);
		}


		public event EventHandler<HostEventArgs> HostRecognized;
		private void OnHostRecognized(HostEventArgs e)
		{
			EventHandler<HostEventArgs> h = HostRecognized;

			if (h != null)
				h(this, e);
		}
		#endregion


		#region Properties
		public bool LimitedMode
		{
			get { return linkTextBox.Enabled; }
			set
			{
				userNameTextBox.ReadOnly = passwordTextBox.ReadOnly =
                renameTextBox.ReadOnly = folderTextBox.ReadOnly =
                    linkTextBox.ReadOnly = value;
				emptyRenameLabel.BackColor = renameTextBox.BackColor;
                folderButton.Enabled = credentialsCheckBox.Enabled = 
                    adapterCombo.Enabled = !value;
			}
		} 


		public string DownloadPath
		{
			get { return folderTextBox.Text; }
			set { folderTextBox.Text = value; }
		}

		public string Link
		{
			get { return linkTextBox.Text; }
			set { linkTextBox.Text = value; }
		}

		public string NewName
		{
			get { return renameTextBox.Text; }
			set { renameTextBox.Text = value; }
		}

        public string Adapter
        {
            get { return ((IconListEntry)adapterCombo.SelectedItem).Value.ToString(); }
            set 
            {
                for (int i = 0; i < adapterCombo.Items.Count; i++)
                {
                    IconListEntry e = adapterCombo.Items[i] as IconListEntry;

                    if (e != null && (String)e.Value == value)
                        adapterCombo.SelectedIndex = i;
                }
            }
        }

		public Credentials Credentials
		{
			get
			{
				return !credentialsCheckBox.Checked || String.IsNullOrEmpty(userNameTextBox.Text) ? null : 
					new Credentials(userNameTextBox.Text, passwordTextBox.Text);
			}
			set
			{
				credentialsCheckBox.Checked = value != null && !String.IsNullOrEmpty(value.UserName);

				if (credentialsCheckBox.Checked)
				{
					userNameTextBox.Text = value.FullUserName;
					passwordTextBox.Text = value.Password;
				}				
			}
		}


		public string LinksCaption
		{
			get { return linkLabel.Text; }
			set { linkLabel.Text = value; }
		}

		public bool DisableAutoUpdate { get; set; }

		public UrlAnalyzer UrlAnalyzer { get; private set; }
		#endregion		

        private void GeneralPropertiesControl_Load(object sender, EventArgs e)
        {
            foreach (Control c in panel.Controls)
                if (!String.IsNullOrEmpty(c.Text))
                    c.Text = StringTable.GetString(c.Text);
        }
	}


	public sealed class UpdateLinksEventArgs : EventArgs
	{
		public UpdateLinksEventArgs(List<DownloadProperties> properties)
		{
			Properties = properties;
		}


		public List<DownloadProperties> Properties { get; private set; }
	}


	public sealed class HostEventArgs : EventArgs
	{
		public HostEventArgs(Host host)
		{
			Host = host;
		}


		public Host Host { get; private set; }
	}
}
