﻿using System;
using System.Windows.Forms;
using System.Drawing;
using WideStream.Workspace;
using WideStream.Workspace.ComponentModel;
using WideStream.DownloadManager.Configuration;
using WideStream.Configuration;
using System.Diagnostics;

namespace WideStream.DownloadManager.Presentation
{
	public sealed class NotificationManager
	{
		#region Construction
		private const int ANIMATION_TIME = 4000;

		public NotificationManager(Main mainForm)
		{
			MainForm = mainForm;
		}
		#endregion


		#region Methods
		public void ShowNotification(Notifications type, string text)
		{
			bool condition = MainForm != null && (!MainForm.Visible || MainForm.WindowState == FormWindowState.Minimized ||
				MainForm.CurrentPage == null || MainForm.CurrentPage.PageType != Page.DownloadManager) &&
				(Config.Query<NotificationOptions>().EnabledNotifications & type) == type;

			if (condition && MainForm.IsHandleCreated && !MainForm.IsDisposed)
				MainForm.Invoke((MethodInvoker)delegate { InternalShowNotification(type, text); });
		}
		

		private void InternalShowNotification(Notifications type, string text)
		{
			NotificationOptions opt = Config.Query<NotificationOptions>();
			NotificationWindow p = new NotificationWindow();
			p.Type = type;
			p.DownloadDisplayName = text;
			var x = Screen.PrimaryScreen.WorkingArea.X + Screen.PrimaryScreen.WorkingArea.Width - p.Width;
			var y = Screen.PrimaryScreen.WorkingArea.Y + Screen.PrimaryScreen.WorkingArea.Height - 
				p.Height - (p.Height * PositionOffset);
			p.Location = new Point(x, y);
			p.Show(MainForm);
			p.FormClosed += (o, e) => PositionOffset--;
			p.ShowDownloadPage += (o, e) => { MainForm.DisplayForm(); MainForm.ChangeCurrentPage(Page.DownloadManager); };
			PositionOffset++;
			Timer timer = new Timer();
			MethodInvoker func = () => { timer.Stop(); p.Close(); };
			int waitTime = (Int32)opt.DisplayTime - ANIMATION_TIME;
			
			if (opt.UseFadeIn)
			{
				int animeTime = 20;
				double opacity = .0D;
				double change = .01D;
				timer.Interval = 20;

				timer.Tick += (o, e) =>
				{
					if (p.IsDisposed)
						timer.Stop();
					else if (p.Pause)
						p.Opacity = 1.0D;
					else
					{
						timer.Interval = animeTime;
						opacity += change;

						if (opacity >= 1.0D)
						{
							change = -.01D;
							timer.Interval = (Int32)opt.DisplayTime - ANIMATION_TIME;
						}
						else if (opacity <= .0D)
							func();

						p.Opacity = opacity;
					}
				};
			}
			else
			{
				p.Opacity = 1.0D;
				timer.Interval = (Int32)opt.DisplayTime;
				timer.Tick += (o, e) => func();				
			}

			timer.Start();
		}
		#endregion


		#region Properties
		public int PositionOffset { get; private set; }

		public Main MainForm { get; private set; }
		#endregion
	}
}
