﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using WideStream.Configuration;
using WideStream.DownloadManager.Configuration;
using WideStream.Startup.Resources;

namespace WideStream.DownloadManager.Presentation
{
    public partial class NotificationWindow : Form
	{
		#region Construction
		private const int WM_RBUTTONDOWN = 0x0204;
		private const int WM_LBUTTONDOWN = 0x0201;
		private const int WM_MBUTTONDOWN = 0x0207;
        private const int WM_MOUSEMOVE = 0x0200;
        private const int WM_MOUSELEAVE = 0x2A3;
		
		public NotificationWindow()
        {
            InitializeComponent();
		}
		#endregion


		#region Methods
		protected override void OnPaint(PaintEventArgs e)
        {
			Color sourceColor = GetBackgroundColor();			
			Color endColor = Color.FromArgb(10, sourceColor);
			
			using (LinearGradientBrush b = new LinearGradientBrush(ClientRectangle, sourceColor, 
				endColor, LinearGradientMode.Vertical))
			{
				e.Graphics.DrawRectangle(Pens.Gray, new Rectangle(ClientRectangle.X + 1, 
					ClientRectangle.Y + 1, ClientRectangle.Width - 2, ClientRectangle.Height - 2));
				e.Graphics.FillRectangle(b, ClientRectangle);
			}

			using (Brush textBursh = new SolidBrush(GetForegroundColor()))
			{
				e.Graphics.DrawString(GetCaption(), Font, textBursh, new Point(5, 5));

				if (Type != Notifications.ManyDownloadAdded)
				{
					string display = DownloadDisplayName.Length > 45 ?
						DownloadDisplayName.Substring(0, 45) + "..." : DownloadDisplayName;
					e.Graphics.DrawString(display, Font, textBursh, new Point(5, 17));
				}

                e.Graphics.DrawString(StringTable.GetString("NotificationWindow.ClickHere"), Font, textBursh,
					new Point(5, Type == Notifications.ManyDownloadAdded ? 17 : 39));
			}

            base.OnPaint(e);
        }


		private Color GetBackgroundColor()
		{
			NotificationOptions opt = Config.Query<NotificationOptions>();
			return Type == Notifications.DownloadAdded || Type == Notifications.ManyDownloadAdded ?
				opt.AddBackColor : Type == Notifications.DownloadCompleted ? opt.CompleteBackColor :
				opt.FailedBackColor;
		}


		private Color GetForegroundColor()
		{
			NotificationOptions opt = Config.Query<NotificationOptions>();
			return Type == Notifications.DownloadAdded || Type == Notifications.ManyDownloadAdded ?
				opt.AddForeColor : Type == Notifications.DownloadCompleted ? opt.CompleteForeColor :
				opt.FailedForeColor;
		}


		private string GetCaption()
		{
			switch (Type)
			{
				case Notifications.DownloadAdded :
                    return StringTable.GetString("NotificationWindow.Added");
				case Notifications.ManyDownloadAdded :
                    return StringTable.GetString("NotificationWindow.AddedMany");
				case Notifications.DownloadFailed:
                    return StringTable.GetString("NotificationWindow.Failed");
				case Notifications.DownloadCompleted:
                    return StringTable.GetString("NotificationWindow.Completed");
				default:
					return String.Empty;
			}
		}


        protected override void WndProc(ref Message m)
        {           
            if (m.Msg == 6)
            {
                if (Owner != null && !Owner.Focused)
                {
                    Owner.Activate();
                    Owner.Text += " ";
                    Owner.Text = Owner.Text.Trim(' ');
                }
            }
            else if (m.Msg == WM_MOUSELEAVE)
                Pause = false;
            else if (m.Msg == WM_MOUSEMOVE)
                Pause = true;
			else if (m.Msg == 512 || m.Msg == 514)
			{
				Close();
				OnShowDownloadPage();
			}
            
            if (!IsDisposed)
                base.WndProc(ref m);
		}
		#endregion


		#region Events
		public event EventHandler ShowDownloadPage;
		private void OnShowDownloadPage()
		{
			EventHandler h = ShowDownloadPage;

			if (h != null)
				h(this, EventArgs.Empty);
		}
		#endregion


		#region Properties
		public Notifications Type { get; set; }

		public string DownloadDisplayName { get; set; }
		
		internal bool Pause { get; set; }
		#endregion
	}
}
