﻿using System;
using System.IO;
using System.Linq;
using System.Collections.Generic;
using System.Xml;
using WideStream.Configuration;
using System.Windows.Forms;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.Core.Network;
using WideStream.DownloadManager.Configuration;
using WideStream.Configuration.ComponentModel;
using WideStream.DownloadManager.Network;
using System.Security;

namespace WideStream.DownloadManager
{
    public sealed class QueueDefinition
    {
        #region Construction
        private QueueDefinition(string fileName)
        {
            FileName = fileName;
            Downloads = new List<DownloadProperties>();
        }
        #endregion


        #region Methods
        public static QueueDefinition OpenQueue(string fileName)
        {
            QueueDefinition def = new QueueDefinition(fileName);
            FileInfo fi = new FileInfo(fileName);

            try
            {
                using (XmlReader reader = XmlReader.Create(fileName))
                    def.ReadFromXml(reader);
            }
            catch (Exception ex)
            {
                if (ex is FileNotFoundException)
                    return null;

                throw ex;
            }

            return def;
        }


        public static QueueDefinition CreateQueue(string fileName)
        {
            QueueDefinition def = new QueueDefinition(fileName);
            FileInfo fi = new FileInfo(fileName);
            return def;
        }


        public void Save()
        {
            SaveAs(FileName);
        }


        public void SaveAs(string fileName)
        {
            FileName = fileName;

            using (XmlWriter writer = XmlWriter.Create(fileName))
                WriteToXml(writer);
        }
        

        public void ReadFromXml(XmlReader reader)
        {
			OptionsSerializer ser = new OptionsSerializer();

			while (reader.ReadToFollowing("Download"))
			{
				DownloadProperties p = new DownloadProperties();
				ser.ReadDataOnly(p, reader);
				Downloads.Add(p);
			}
        }
        
       
        public void WriteToXml(XmlWriter writer)
        {
			writer.WriteStartElement("Downloads");
			writer.WriteAttributeString("Version", "3");
			OptionsSerializer ser = new OptionsSerializer();
			
			foreach (DownloadProperties p in Downloads)
			{
				writer.WriteStartElement("Download");
				ser.WriteDataOnly(p, writer, null);
				writer.WriteEndElement();
			}

			writer.WriteEndElement();
        }
        #endregion


        #region Properties
        public string FileName { get; private set; }

        public List<DownloadProperties> Downloads { get; private set; }
        #endregion
    }
}
