﻿using System;
using System.Windows.Forms;
using WideStream.FtpManager;
using System.ComponentModel;
using System.Drawing;
using WideStream.Workspace.Forms;
using WideStream.Startup.Resources;
using WideStream.Core.Network;

namespace WideStream.FtpManager.Presentation
{
    public sealed class Explorer : IExplorer
    {
        #region Construction
        private DataGridView dataGrid;
        private ImageList imageList;
        private ToolStripLabel label;
        private ContextMenuStrip contextMenu;
        
        public Explorer(ToolStripLabel label, DataGridView dataGrid, ImageList imageList, ContextMenuStrip contextMenu)
        {
            this.label = label;
            this.dataGrid = dataGrid;
            this.imageList = imageList;
            this.contextMenu = contextMenu;
        }
        #endregion


        #region Methods
		public void Clear()
        {
            if (dataGrid != null && !dataGrid.IsDisposed)
            {
                dataGrid.Invoke((MethodInvoker)delegate {
                    dataGrid.Rows.Clear();
                });
            }
        }


        public void ListNode(FileNode node)
        {
            if (dataGrid != null && !dataGrid.IsDisposed && node != null)
            {
                dataGrid.Invoke((MethodInvoker)delegate {
                    label.Image = imageList.Images["normalstate"];
                    label.ToolTipText = String.Empty;
                    DataGridViewRow tr = new DataGridViewRow();
                    tr.ContextMenuStrip = contextMenu;

                    if (node.FileName == "...")
                        tr.Cells.Add(CreateCell(node.FileName, true));
                    else
                    {
                        tr.Cells.AddRange(CreateImageCell(node.FileName,
                                node.IsDirectory ? "folder" : "file"),
                            CreateCell(node.IsDirectory ? StringTable.GetString("Explorer.Directory") :
                                StringTable.GetString("Explorer.File"), false),
                            CreateCell(node.IsDirectory ? String.Empty : CalculateLength(node.Size), false),
                            CreateCell(node.ChangeDate.ToString(), false));
                        tr.Tag = node;
                    }

                    dataGrid.Rows.Add(tr);                    
                });
            }
        }


        public void ListError(string error)
        {
            if (FtpExplorer.Instance.HasParent)
                ListNode(new FileNode { FileName = "..." });

            if (dataGrid != null && !dataGrid.IsDisposed)
            {
                dataGrid.Invoke((MethodInvoker)delegate {
                    label.Image = imageList.Images["errorstate"];
                    label.ToolTipText = error;
                });
            }
        }


        public void FinishListing()
        {
            if (dataGrid != null && !dataGrid.IsDisposed)
            {
                dataGrid.Invoke((MethodInvoker)delegate {
                    if (dataGrid.SortedColumn != null)
                        dataGrid.Sort(dataGrid.SortedColumn, dataGrid.SortOrder == SortOrder.Ascending ?
                            ListSortDirection.Ascending : ListSortDirection.Descending);
                });
            }
        }


        private DataGridViewIconCell CreateImageCell(string message, string imageKey)
        {
            DataGridViewIconCell td = new DataGridViewIconCell();
            td.Value = message;
            td.Style.Alignment = DataGridViewContentAlignment.TopLeft;
            td.Style.WrapMode = DataGridViewTriState.True;
            td.Image = imageList.Images[imageKey];
            return td;
        }


        private DataGridViewTextBoxCell CreateCell(string message, bool bold)
        {
            DataGridViewTextBoxCell td = new DataGridViewTextBoxCell();
            td.Value = message;
            td.Style.Alignment = DataGridViewContentAlignment.TopLeft;
            td.Style.WrapMode = DataGridViewTriState.True;

            if (bold)
            {
                td.Style.Padding = new Padding(8, 0, 0, 0);
                td.Style.Font = new Font(dataGrid.Font, FontStyle.Bold);
            }

            return td;
        }


        private string CalculateLength(float value)
        {
            return new DataSize((Int32)value).ToString(SizeFormat.Regular);
        }
        #endregion
    }
}
