﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using WideStream.Core.ComponentModel;
using WideStream.Core.Runtime;
using WideStream.DownloadManager;
using WideStream.FtpManager;
using WideStream.Workspace;
using WideStream.Configuration;
using WideStream.Workspace.Forms;
using WideStream.DownloadManager.Presentation;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.Core.Network;
using WideStream.Logging;
using WideStream.FtpManager.Configuration;
using WideStream.Workspace.ComponentModel;
using WideStream.HostManager.Configuration;
using WideStream.Configuration.ComponentModel;
using WideStream.DownloadManager.Configuration;
using WideStream.Startup.Resources;
using WideStream.Workspace.State;

namespace WideStream.FtpManager.Presentation
{
    public partial class FtpManagerPage : UserControl, IPage
    {
        #region Construction
        public FtpManagerPage()
        {
            InitializeComponent();
            LocalizationManager.Localize(this);
            ThemeManager.ApplyTheme(this);                       
            Dock = DockStyle.Fill;
        }
        #endregion


        #region Methods
        public bool CanClose()
        {
            return true;
        }


        public void Open()
        {
            if (_activeLog != null)
                _activeLog.Active = true;

            if (_activeLog == null)
            {
                _activeLog = new GridLog(_logControl);
                _activeLog.Active = true;
            }            
        }


        public void Initialize(Main parent, DataGridView logControl)
        {
            _logControl = logControl;
            errorLabel.Image = imageList.Images["normalstate"];
            openButton.Image = imageList.Images["go"];
            CurrentCredentials = CurrentCredentials;
            RefreshHistory();
        }
        #endregion


        #region Properties
        public Page PageType
        {
            get { return Page.FtpExplorer; }
        }

        public MenuStrip MainMenu
        {
            get { return ftpMenu; }
        }

        public new ContextMenuStrip ContextMenu
        {
            get { return contextMenu; }
        }


        public ContextMenuStrip ColumnsMenu
        {
            get { return columnsMenu; }
        }

        private ILog _activeLog;
        public ILog ActiveLog
        {
            get { return _activeLog; }
        }


        private DataGridView _logControl;
        public DataGridView LogControl
        {
            get { return _logControl; }
            set { _logControl = value; }
        }




        public DataGridView DisplayControl
        {
            get { return ftpGrid; }
        }


        public FileNode SelectedFile
        {
            get
            {
                return ftpGrid.SelectedRows.Count != 1 ? null :
                    (FileNode)ftpGrid.SelectedRows[0].Tag;
            }
        }


        public ToolStrip ToolBar
        {
            get { return toolStrip; }
        }


        public List<FileNode> SelectedFiles
        {
            get
            {
                List<FileNode> props = new List<FileNode>();

                if (ftpGrid.SelectedRows.Count > 0)
                {
                    foreach (DataGridViewRow tr in ftpGrid.SelectedRows)
                        if (tr.Tag != null)
                            props.Add((FileNode)tr.Tag);
                }

                return props;
            }
        }


        public string CurrentStatus
        {
            get
            {
                if (FtpExplorer.Instance.IsWorking)
                {
                    progressImage.Visible = true;
                    return "Connecting to " + FtpExplorer.Instance.Properties.Uri.ToString();                    
                }
                else
                    progressImage.Visible = false;

                return String.Empty;
            }
        }
        
        private Credentials _currentCredentials;
        public Credentials CurrentCredentials
        {
            get { return _currentCredentials; }
            private set
            {
                _currentCredentials = value;
                credentialsButton.Image = _currentCredentials != null ? imageList.Images["hascred"] :
                    imageList.Images["nocred"];                
            }
        }
        #endregion

    

        private void credentialsButton_Click(object sender, EventArgs e)
        {
            Uri uri;
            CredentialsDialog d = new CredentialsDialog();
            bool hasInlineCred = false;

            if (Uri.TryCreate(addressCombo.Text, UriKind.Absolute, out uri) && !String.IsNullOrEmpty(uri.UserInfo))
            {
                string[] arr = uri.UserInfo.Split(new char[] { ':' }, StringSplitOptions.RemoveEmptyEntries);
                Credentials cred = new Credentials(arr[0], arr.Length > 1 ? arr[1] : String.Empty);
                d.Credentials = cred;
                hasInlineCred = true;
            }
            else if (CurrentCredentials != null)
                d.Credentials = CurrentCredentials;
            
            if (d.ShowDialog(this) == DialogResult.OK)
            {
                CurrentCredentials = d.Credentials;

                foreach (HistoryItem h in addressCombo.Items)
                    if (String.Compare(h.Url.OriginalString, addressCombo.Text, true) == 0)
                        h.Credentials = d.Credentials;
            }

            if (hasInlineCred)
                addressCombo.Text = uri.ToString().Replace(uri.UserInfo + "@", String.Empty);
        }

        private void openButton_Click(object sender, EventArgs e)
        {
            Uri uri = GetUri();

            if (uri == null)
                return;
            
            DirectoryProperties props = new DirectoryProperties { Uri = uri, Credentials = _currentCredentials };
            FtpExplorer.Instance.Properties = null;
            ListDirectory(props);

            HistoryItem h = new HistoryItem(uri,
                _currentCredentials != null ? (String.IsNullOrEmpty(_currentCredentials.Domain) ? 
                    _currentCredentials.UserName : String.Concat(_currentCredentials.Domain, "\\", 
                        _currentCredentials.UserName)) : String.Empty,
                _currentCredentials != null ? _currentCredentials.Password : String.Empty);
            AddHistory(h);            
        }


		private void AddHistory(HistoryItem h)
        {
			ExplorerHistoryOptions conf = Config.Query<ExplorerHistoryOptions>();
            bool add = false;

            if ((conf.HistorySaveRule == HistorySaveRule.Always) || 
                (conf.HistorySaveRule == HistorySaveRule.OnlyDifferentLinks && !conf.History.Contains(h)))
                add = true;

            if (conf.HistorySaveRule == HistorySaveRule.OnlyDifferentDomains)
            {
                string currentHostName = h.Url.Host;
                add = true;

                foreach (HistoryItem oh in conf.History)
                    if (String.Compare(oh.Url.Host, currentHostName, true) == 0)
                        add = false;
            }
            
            if (add)
            {
                conf.History.Add(h);
                addressCombo.Items.Add(h);         
            }

			if (Config.Query<ExplorerHistoryOptions>().History.Count > (Int32)Config.Query<ExplorerHistoryOptions>().HistorySize)
            {
				Config.Query<ExplorerHistoryOptions>().History.RemoveAt(0);
                addressCombo.Items.RemoveAt(0);
            }
        }



        public void RefreshHistory()
        {
            addressCombo.Items.Clear();

			foreach (HistoryItem h in Config.Query<ExplorerHistoryOptions>().History)
                addressCombo.Items.Add(h);
        }


        public Uri GetUri()
        {
            Uri uri;

            if (!Uri.TryCreate(addressCombo.Text, UriKind.Absolute, out uri) || 
                String.Compare(uri.Scheme, "ftp", true) != 0)
                return null;

            return uri;
        }


        private void ListDirectory(DirectoryProperties properties)
        {
            if (properties != null)
            {
                FtpExplorer.Instance.ListDirectory(properties, _activeLog, new Explorer(errorLabel, ftpGrid, imageList, contextMenu));
                addressCombo.Text = properties.Uri.ToString();
            }
            else
            {
                addressCombo.Text = FtpExplorer.Instance.ListParentDirectory().ToString();
            }
        }


        private void addressCombo_TextChanged(object sender, EventArgs e)
        {
            Uri uri;
            Host h = null;

			if (Uri.TryCreate(addressCombo.Text, UriKind.Absolute, out uri) && !String.IsNullOrEmpty(uri.UserInfo))
			{
				string[] arr = uri.UserInfo.Split(':');
				CurrentCredentials = new Credentials(arr[0], arr.Length > 1 ? arr[1] : String.Empty);
			}
			
            if (uri != null && (h = Config.Query<HostManagerOptions>().Hosts.FirstOrDefault(
                he => he.Slike(uri.Host.ToLower()))) != null)
				CurrentCredentials = h.Credentials;
        }

        private void ftpGrid_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            PerformNodeAction();   
        }


        private void PerformNodeAction()
        {
            if (ftpGrid.SelectedRows.Count == 1)
            {
                FileNode node = ftpGrid.SelectedRows[0].Tag as FileNode;

                if (node == null || node.IsDirectory)
                    ListDirectory(node == null ? null :
                        new DirectoryProperties { Uri = node.FileUri, Credentials = _currentCredentials });
                else if (node != null && !node.IsDirectory)
                    AddDownloads();
            }
            else if (ftpGrid.SelectedRows.Count > 1)
                AddDownloads();
        }



        private void downloadFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AddDownloads();
        }


        private void AddDownloads()
        {
            ConfigureDownloadDialog d = new ConfigureDownloadDialog(false);
            UrlAnalyzer analyzer = new UrlAnalyzer(true);

            foreach (FileNode f in SelectedFiles)
            {
                if (!f.IsDirectory)
                {
                    DownloadProperties dp = new DownloadProperties();
                    analyzer.InitializeDownload(dp, null, f.FileUri);

                    if (_currentCredentials != null)
                        dp.Credentials = _currentCredentials;

                    d.Downloads.Add(dp);
                }
            }

            d.SingleLinkMode = d.Downloads.Count < 2;

            if (d.Downloads.Count > 0 && d.ShowDialog(this) == DialogResult.OK)
            {
				DownloadPage page;

				if (Config.Query<FtpManagerOptions>().DownloadAction == FtpDownloadAction.StayInExplorer)
					page = (DownloadPage)((Main)ParentForm).LoadPage(Page.DownloadManager);
				else
					page = (DownloadPage)((Main)ParentForm).ChangeCurrentPage(Page.DownloadManager);
				
                page.AddDownloads(d.EnumerateDownloads(),
                    (Int32)Config.Query<DownloadManagerDisplayOptions>().QueueScroll > 0);
            }
        }

        private void progressImage_Click(object sender, EventArgs e)
        {
            FtpExplorer.Instance.Abort();
        }

        private void addressCombo_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                openButton_Click(this, EventArgs.Empty);
        }

        private void backButton_Click(object sender, EventArgs e)
        {
            ListDirectory(null);
        }

        private void refreshButton_Click(object sender, EventArgs e)
        {
            if (_currentCredentials != null)
                FtpExplorer.Instance.Properties.Credentials = _currentCredentials;

            FtpExplorer.Instance.Refresh();
        }


        private void copyDownloadURLToolStripMenuItem_Click(object sender, EventArgs e)
        {
            StringBuilder sb = new StringBuilder();

            foreach (DataGridViewRow tr in ftpGrid.SelectedRows)
            {
                if (tr.Tag != null)
                {
                    sb.Append(((FileNode)tr.Tag).FileUri.ToString());
                    sb.Append("\r\n");
                }
            }

            Clipboard.SetText(sb.ToString());
        }

        private void copyallDownloadsURLsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            StringBuilder sb = new StringBuilder();

            foreach (DataGridViewRow tr in ftpGrid.Rows)
            {
                if (tr.Tag != null)
                {
                    sb.Append(((FileNode)tr.Tag).FileUri.ToString());
                    sb.Append("\r\n");
                }
            }

            Clipboard.SetText(sb.ToString());
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PerformNodeAction();
        }


        private void ftpGrid_CellMouseClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
                ftpGrid.Rows[e.RowIndex].Selected = true;
        }

        private void contextMenu_Opening(object sender, CancelEventArgs e)
        {
            if (ftpGrid.SelectedRows.Count > 1)
                openToolStripMenuItem.Text = StringTable.GetString("ExplorerPage.DefaultDownload");
            else if (ftpGrid.SelectedRows.Count == 1 && ftpGrid.SelectedRows[0].Tag == null)
                openToolStripMenuItem.Text = StringTable.GetString("ExplorerPage.DefaultParent");
            else if (ftpGrid.SelectedRows.Count == 1 && ftpGrid.SelectedRows[0].Tag != null &&
                ((FileNode)ftpGrid.SelectedRows[0].Tag).IsDirectory)
                openToolStripMenuItem.Text = StringTable.GetString("ExplorerPage.DefaultOpen");
            else if (ftpGrid.SelectedRows.Count == 1 && ftpGrid.SelectedRows[0].Tag != null &&
                !((FileNode)ftpGrid.SelectedRows[0].Tag).IsDirectory)
                openToolStripMenuItem.Text = StringTable.GetString("ExplorerPage.DefaultDownloadFile");
            else
                openToolStripMenuItem.Text = StringTable.GetString("ExplorerPage.DefaultNo");
        }

        private void addressCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            HistoryItem h = (HistoryItem)addressCombo.SelectedItem;
			CurrentCredentials = h.Credentials;
        }

		private void ftpGrid_EnterPressed(object sender, EventArgs e)
		{
			openToolStripMenuItem_Click(sender, e);
		}

        private void columnsMoreItem_Click(object sender, EventArgs e)
        {
            ((Main)this.ParentForm).ShowOptionsDialog(StringTable.GetString("FtpManagerOptions.ShortName"));
        }

        private void columnsMenu_Opening(object sender, CancelEventArgs e)
        {
            var config = Config.Query<FtpManagerOptions>();

            if (columnsMenu.Items.Count < 3)
            {
                foreach (var v in config.ExplorerColumns.GetDisplayNames())
                {
                    var mi = new ToolStripMenuItem(v.Name);
                    mi.Tag = v.Value;
                    mi.Click += (o, ev) =>
                    {
                        if (((ToolStripMenuItem)o).Checked)
                            config.ExplorerColumns ^= (ExplorerColumns)((ToolStripMenuItem)o).Tag;
                        else
                            config.ExplorerColumns |= (ExplorerColumns)((ToolStripMenuItem)o).Tag;
                    };
                    columnsMenu.Items.Insert(columnsMenu.Items.Count - 2, mi);
                }
            }

            foreach (ToolStripItem mi in columnsMenu.Items)
            {
                if (mi.Tag != null)
                {
                    ExplorerColumns val = (ExplorerColumns)mi.Tag;
                    ((ToolStripMenuItem)mi).Checked = (config.ExplorerColumns & val) == val;
                }
            }
        }


    }
}
