﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using WideStream.Configuration.ComponentModel;
using WideStream.Core.Network;
using System.Reflection;
using WideStream.Startup.Resources;

namespace WideStream.FtpManager.Presentation
{
	public sealed class HistoryList : Panel, IEditor
	{
		#region Construction
		private ListBox listBox;
		private Button button;
		private List<HistoryItem> hosts;

		public HistoryList()
		{
			Initialize();
		}
		#endregion


		#region Methods
		private void Initialize()
		{
			TableLayoutPanel panel = new TableLayoutPanel();
			panel.RowCount = 2;
			panel.ColumnCount = 2;
			panel.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 100F));
			panel.ColumnStyles.Add(new ColumnStyle(SizeType.Absolute, 75F));
			panel.RowStyles.Add(new RowStyle(SizeType.Percent, 80F));
			panel.RowStyles.Add(new RowStyle(SizeType.Absolute, 26F));
			panel.Dock = DockStyle.Fill;
			
			listBox = new ListBox();
			listBox.Margin = new Padding(0, 0, 0, 0);
			listBox.Dock = DockStyle.Fill;
			panel.Controls.Add(listBox, 0, 0);
			panel.SetColumnSpan(listBox, 2);

			button = new Button();
			button.Margin = new Padding(0, 0, 0, 0);
			button.Size = new Size(75, 23);
			button.Click += (o, e) => ClearHistory();
			button.Text = StringTable.GetString("ExplorerHistoryList.Clear");
			panel.Controls.Add(button, 1, 1);
			Controls.Add(panel);
		}


		private void ClearHistory()
		{
			listBox.Items.Clear();
			hosts.Clear();
			OnValueChanged(new EditorEventArgs(Value));
		}


		public void SetStandardValues(object value)
		{
			hosts = (List<HistoryItem>)value;

			foreach (HistoryItem h in hosts)
				listBox.Items.Add(h.Url.ToString());
		}
		#endregion


		#region Properties
		public object Value
		{
			get { return hosts; }
			set { hosts = (List<HistoryItem>)value; }
		}

		public PropertyInfo RelatedProperty { get; set; }

        public bool IsDirty { get; private set; }
        #endregion
		

		#region Events
		public event EventHandler<EditorEventArgs> ValueChanged;
		private void OnValueChanged(EditorEventArgs e)
		{
			EventHandler<EditorEventArgs> h = ValueChanged;

			if (h != null)
				h(this, e);
		}
		#endregion
	}
}
