﻿using System;
using System.Collections.Generic;
using WideStream.Configuration.ComponentModel;
using WideStream.Core.Runtime;
using WideStream.DownloadManager.Configuration;
using WideStream.DownloadManager.Network;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.Core.Network;

namespace WideStream.HostManager.Configuration
{
	[Options]
    public sealed class Host : Options, IStringComparable
    {
        #region Construction
        public Host()
        {
			ProtocolOptions = new List<Options>();
			GeneralOptions = new DownloadProcessOptions();

			foreach (DownloadProtocol t in DownloadProtocolManager.EnumerateDownloadTypes())
				ProtocolOptions.Add(t.Options.Clone());
        }
        #endregion


        #region Properties
		[Editor("Host.HostName", ColSpan = true, Description = "Host.HostNameDescription")]
        public string HostName { get; set; }

		[Editor("Host.Description", Editor = PropertyEditor.MultilineTextBox, ColSpan = true, Height = 80)]
		public string Description { get; set; }

		[Editor(SkipLabel = true, ColSpan = true, Height = 100)]
		public Credentials Credentials { get; set; }

		[SaveRule(Rule = SaveRule.NestedOptions, OnlyEnabled = true)]
		public DownloadProcessOptions GeneralOptions { get; set; }

		[SaveRule(Rule = SaveRule.Collection, OnlyEnabled = true)]
		public List<Options> ProtocolOptions { get; private set; }
      
		[SaveRule(Ignore = true)]
		string IStringComparable.StringValue
		{
			get { return HostName; }
		}
		#endregion
	}
}
