﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using WideStream.Core.ComponentModel;
using WideStream.DownloadManager;
using WideStream.FtpManager;
using WideStream.Workspace;
using WideStream.Configuration;
using WideStream.Workspace.Forms;
using WideStream.DownloadManager.Presentation;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.Core.Network;
using WideStream.Logging;
using WideStream.Configuration.ComponentModel;
using WideStream.FtpManager.Configuration;
using WideStream.Workspace.ComponentModel;
using WideStream.DownloadManager.Configuration;
using WideStream.DownloadManager.Network.Protocols.Http;
using WideStream.DownloadManager.Network.Protocols.Ftp;
using WideStream.DownloadManager.Network;
using System.Collections;
using WideStream.HostManager.Configuration;
using WideStream.Startup.Resources;
using WideStream.Workspace.State;

namespace WideStream.HostManager.Presentation
{
    public partial class HostManagerPage : UserControl, IPage
    {
        #region Construction
		public HostManagerPage()
        {
            InitializeComponent();
            LocalizationManager.Localize(this);
            ThemeManager.ApplyTheme(this);            
            Dock = DockStyle.Fill;
        }
        #endregion


        #region Methods
        public bool CanClose()
        {
            return true;
        }


        public void Open()
        {
            
        }


        public void Initialize(Main parent, DataGridView logControl)
        {
			foreach (Host h in Config.Query<HostManagerOptions>().Hosts)
				hostGrid.Rows.Add(CreateRow(h));

            _logControl = logControl;
        }


		private DataGridViewRow CreateRow(Host h)
		{
			DataGridViewRow tr = new DataGridViewRow();
			tr.Cells.Add(CreateCell(h.HostName));
			tr.Cells.Add(CreateCell(h.Description));
			tr.Cells.Add(CreateCell(h.GeneralOptions.Timeout.ToDisplayString()));
			tr.Cells.Add(CreateCell(h.GeneralOptions.ErrorLimit.ToDisplayString()));
			tr.Cells.Add(CreateCell(h.GeneralOptions.Priority.ToDisplayString()));
			tr.Cells.Add(CreateCell(h.GeneralOptions.MaximumDownloadConnections.ToDisplayString()));
			tr.Tag = h;
            tr.ContextMenuStrip = contextMenu;
			return tr;
		}


        private void UpdateRow(DataGridViewRow tr, Host h)
        {
            tr.Cells[0].Value = h.HostName;
            tr.Cells[1].Value = h.Description;
            tr.Cells[2].Value = h.GeneralOptions.Timeout.ToDisplayString();
            tr.Cells[3].Value = h.GeneralOptions.ErrorLimit.ToDisplayString();
            tr.Cells[4].Value = h.GeneralOptions.Priority.ToDisplayString();
            tr.Cells[5].Value = h.GeneralOptions.MaximumDownloadConnections.ToDisplayString();
        }


		private DataGridViewCell CreateCell(string value)
		{
			DataGridViewCell td = new DataGridViewTextBoxCell();
			td.Value = value;
			return td;
		}
        #endregion


        #region Properties
        public Page PageType
        {
            get { return Page.HostManager; }
        }

        public MenuStrip MainMenu
        {
            get { return hostMenu; }
        }

        public new ContextMenuStrip ContextMenu
        {
            get { return contextMenu; }
        }

        public ContextMenuStrip ColumnsMenu
        {
            get { return columnsMenu; }
        }


        public ILog ActiveLog
        {
            get { return null; }
        }


        private DataGridView _logControl;
        public DataGridView LogControl
        {
            get { return _logControl; }
            set { _logControl = value; }
        }
		

        public DataGridView DisplayControl
        {
            get { return hostGrid; }
        }


        public ToolStrip ToolBar
        {
            get { return toolStrip; }
        }


        public string CurrentStatus
        {
            get
            {
               
                return String.Empty;
            }
        }
        #endregion


		private void addNewHostToolStripMenuItem_Click(object sender, EventArgs e)
		{
			Host h = new Host();
			ShowHostDialog(h, false);
		}

		private void hostPropertiesToolStripMenuItem_Click(object sender, EventArgs e)
		{
			if (hostGrid.SelectedRows.Count > 0)
			{
				Host h = (Host)hostGrid.SelectedRows[0].Tag;
				ShowHostDialog(h, true);
			}
		}


		private void ShowHostDialog(Host h, bool replace)
		{
			HostPropertiesDialog d = new HostPropertiesDialog();
			d.Host = h;

			if (d.ShowDialog(this) == DialogResult.OK)
			{
                if (replace)
				{
					int index = hostGrid.SelectedRows[0].Index;
                    var tr = hostGrid.SelectedRows[0];
                    UpdateRow(tr, h);
				}
				else
				{
					var hosts = Config.Query<HostManagerOptions>().Hosts;

					if (hosts.Count(ht => String.Compare(ht.HostName, d.Host.HostName, true) == 0) > 0)
					{
                        MessageBox.Show(this, String.Format(StringTable.GetString("HostManager.HostAlreadyRegistered"), h.HostName),
							Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning);
						ShowHostDialog(d.Host, replace);
					}
					else
					{
						hosts.Add(d.Host);
						hostGrid.Rows.Add(CreateRow(d.Host));                        
					}
				}
			}
		}

		private void hostGrid_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
		{
			hostPropertiesToolStripMenuItem_Click(sender, EventArgs.Empty);
		}

		private void removeSelectedHostToolStripMenuItem_Click(object sender, EventArgs e)
		{
			foreach (DataGridViewRow tr in hostGrid.SelectedRows)
			{
				Host h = (Host)tr.Tag;
				hostGrid.Rows.Remove(tr);
				Config.Query<HostManagerOptions>().Hosts.Remove(h);
			}
		}

		private void copyHostNameToolStripMenuItem_Click(object sender, EventArgs e)
		{
			CopyHostNames(hostGrid.SelectedRows);
		}

		private void copyAllHostNamesToolStripMenuItem_Click(object sender, EventArgs e)
		{
			CopyHostNames(hostGrid.Rows);
		}


		private void CopyHostNames(IList rows)
		{
			StringBuilder sb = new StringBuilder();

			foreach (DataGridViewRow r in rows)
				sb.AppendLine(((Host)r.Tag).HostName);

			Clipboard.SetText(sb.ToString());
		}

		private void hostGrid_EnterPressed(object sender, EventArgs e)
		{
			hostPropertiesToolStripMenuItem_Click(sender, e);
		}

        private void columnsMoreItem_Click(object sender, EventArgs e)
        {
            ((Main)this.ParentForm).ShowOptionsDialog("Host Manager");
        }

        private void columnsMenu_Opening(object sender, CancelEventArgs e)
        {
            var config = Config.Query<HostManagerOptions>();

            if (columnsMenu.Items.Count < 3)
            {
                foreach (var v in config.DisplayColumns.GetDisplayNames())
                {
                    var mi = new ToolStripMenuItem(v.Name);
                    mi.Tag = v.Value;
                    mi.Click += (o, ev) =>
                    {
                        if (((ToolStripMenuItem)o).Checked)
                            config.DisplayColumns ^= (HostManagerColumns)((ToolStripMenuItem)o).Tag;
                        else
                            config.DisplayColumns |= (HostManagerColumns)((ToolStripMenuItem)o).Tag;
                    };
                    columnsMenu.Items.Insert(columnsMenu.Items.Count - 2, mi);
                }
            }

            foreach (ToolStripItem mi in columnsMenu.Items)
            {
                if (mi.Tag != null)
                {
                    HostManagerColumns val = (HostManagerColumns)mi.Tag;
                    ((ToolStripMenuItem)mi).Checked = (config.DisplayColumns & val) == val;
                }
            }
        }
    }
}
