﻿using System;
using System.Windows.Forms;
using WideStream.Configuration;
using WideStream.Core.Network;
using WideStream.Configuration.ComponentModel;
using WideStream.HostManager.Configuration;
using WideStream.Startup.Resources;

namespace WideStream.HostManager.Presentation
{
    public partial class HostPropertiesDialog : Form
    {
        #region Construction
        public HostPropertiesDialog()
        {
			Renderer = new OptionPageRenderer();
			Renderer.SkipHeader = true;
            InitializeComponent();            
        }
        #endregion


        #region Methods
        private void HostPropertiesDialog_Load(object sender, EventArgs e)
		{
            Renderer.AllowDefault = false;
            hostTabPage.Padding = new Padding(10, 3, 10, 10);
			hostTabPage.Controls.Add(Renderer.RenderOptions(Host));
            Renderer.AllowDefault = true;
            AddPage(Host.GeneralOptions, StringTable.GetString("HostProperties.GeneralPage"));
                       
            foreach (Options opt in Host.ProtocolOptions)
			{
				var attr = OptionsAttribute.GetAttribute(opt.GetType());
				AddPage(opt, attr.ShortName);
			}

            Text = StringTable.GetString(Text);
            hostTabPage.Text = StringTable.GetString(hostTabPage.Text);
            acceptButton.Text = StringTable.GetString(acceptButton.Text);
            cancelButton.Text = StringTable.GetString(cancelButton.Text);
		}


		private void AddPage(Options options, string name)
		{
			TabPage tab = null;

			foreach (TabPage p in tabControl.TabPages)
				if (p.Text == name)
				{
					tab = p;
					tab.Controls.Clear();
				}

			if (tab == null)
			{
				tab = new TabPage();
				tab.Padding = new Padding(10, 3, 10, 10);
				tab.Text = StringTable.GetString(name);
				tab.UseVisualStyleBackColor = true;
				tabControl.TabPages.Add(tab);
			}

			Control control = Renderer.RenderOptions(options);
			tab.Controls.Add(control);
		}


        private void HostPropertiesDialog_FormClosing(object sender, FormClosingEventArgs e)
        {
            Renderer.ApplyChanges();
        }
        #endregion


		#region Properties
		public Host Host { get; set; }

		internal IOptionPageRenderer Renderer { get; private set; }
		#endregion		
	}
}
