﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using Microsoft.Win32;
using WideStream.Configuration;
using System.Windows.Forms;

namespace WideStream.Integration.InternetExplorer
{
	public sealed class InternetExplorerIntegration : IBrowserIntegration
	{
		#region Construction
		private const string REGISTRY_KEY = @"HKEY_CURRENT_USER\Software\Microsoft\Internet Explorer\MenuExt\Download in WideStream";

		public InternetExplorerIntegration()
		{

		}
		#endregion


		#region Methods
		public bool IsInstalled()
		{
			string fileName;
			return (fileName = Registry.GetValue(REGISTRY_KEY, null, null) as String) != null &&
				new FileInfo(fileName).Exists;
		}


		public bool IsAvailable()
		{
			return true;
		}


		public bool Install()
		{
			string scriptSource;

			using (Stream stream = typeof(InternetExplorerIntegration).Assembly.
				GetManifestResourceStream("WideStream.Integration.InternetExplorer.IntegrationScript.inc"))
			using (StreamReader sr = new StreamReader(stream))
				scriptSource = sr.ReadToEnd().Replace("%LOC%", Application.ExecutablePath.Replace(@"\", @"\\"));

			FileInfo fi = new FileInfo(Path.Combine(Config.InitializeDefaultDirectory().FullName, "IEINT.htm"));

			using (StreamWriter sw = File.CreateText(fi.FullName))
				sw.Write(scriptSource);

			Registry.SetValue(REGISTRY_KEY, null, fi.FullName);
			Registry.SetValue(REGISTRY_KEY, "contexts", 32);
			return true;
		}


		public bool Uninstall()
		{
			Registry.CurrentUser.DeleteSubKey(REGISTRY_KEY.Substring(REGISTRY_KEY.IndexOf("\\") + 1));
			return true;
		}


		public bool Repair()
		{
			return Install();
		}
		#endregion
	}
}
