﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using WideStream.Startup.Resources;

namespace WideStream.Integration.Presentation
{
	public partial class BrowserDataControl : UserControl
	{
		public BrowserDataControl()
		{
			InitializeComponent();
		}


		private void UpdateControl()
		{
			browserIcon.Image = Integration.BrowserImage;
			browserNameLabel.Text = Integration.BrowserName;
			installButton.Visible = !(uninstallButton.Visible = repairButton.Visible =
				Integration.Addin.IsInstalled());
            Enabled = Integration.Addin.IsAvailable();
		}


		private void Fail(string act)
		{
            MessageBox.Show(this, String.Format(StringTable.GetString(act),
				Integration.BrowserName), ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning);
		}


		protected override void OnPaint(PaintEventArgs e)
		{
			e.Graphics.DrawRectangle(SystemPens.ControlDark, 0, 0, ClientRectangle.Width - 1, ClientRectangle.Height - 1);
			base.OnPaint(e);
		}


        private IntegrationAddin _integration;
		public IntegrationAddin Integration
		{
			get { return _integration; }
			set
			{
				_integration = value;
				UpdateControl();
			}
		}

		private void installButton_Click(object sender, EventArgs e)
		{
            if (!Integration.Addin.Install())
                Fail("Integration.InstallFail");
			else
				UpdateControl();
		}

		private void uninstallButton_Click(object sender, EventArgs e)
		{
            if (!Integration.Addin.Uninstall())
                Fail("Integration.UninstallFail");
			else
				UpdateControl();
		}

		private void repairButton_Click(object sender, EventArgs e)
		{
            if (!Integration.Addin.Repair())
                Fail("Integration.RepairFail");
			else
			{
				UpdateControl();
                MessageBox.Show(this, StringTable.GetString("Integration.RepairComplete"), ProductName);
			}
		}

		private void detailLink_Click(object sender, EventArgs e)
		{
			MessageBox.Show(this, Integration.Description, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
		}


		public new bool Enabled
		{
			get { return installButton.Enabled && uninstallButton.Enabled && repairButton.Enabled; }
			private set { installButton.Enabled = uninstallButton.Enabled = repairButton.Enabled = value; }
		}


        private void BrowserDataControl_Load(object sender, EventArgs e)
        {
            detailLink.Text = StringTable.GetString(detailLink.Text);
            installButton.Text = StringTable.GetString(installButton.Text);
            repairButton.Text = StringTable.GetString(repairButton.Text);
            uninstallButton.Text = StringTable.GetString(uninstallButton.Text);
        }
	}
}
