﻿using System;
using System.Drawing;
using WideStream.Core.ComponentModel;
using WideStream.Configuration.ComponentModel;

namespace WideStream.Logging.Configuration
{
    [Options(Category = "LogOptions.Category", LongName = "LogOptions.LongName", ShortName = "LogOptions.ShortName")]
	public sealed class LogOptions : Options
	{
		#region Construction
		private object syncRoot = new Object();

		public LogOptions()
		{
			LoggingMode = LoggingMode.Verbose;
			InformationColor = Color.Black;
			WarningColor = Color.Blue;
			VerboseColor = Color.Black;
			ErrorColor = Color.Red;
			SuccessColor = Color.Green;
			ImportantColor = Color.Blue;
			BackgroundColor = SystemColors.Window;
			MultilineBackgroundColor = Color.WhiteSmoke;
			LogColumns = LogColumns.Number | LogColumns.Time;
			ScrollRule = LogScroll.Always;
			AllowRegroupColumns = AllowResizeColumns = false;
		}
		#endregion


		#region Properties
        [Editor("LogOptions.ScrollRule")]
		public LogScroll ScrollRule { get; set; }

        [Editor("LogOptions.LoggingMode")]
		public LoggingMode LoggingMode { get; set; }

        [Editor("LogOptions.InformationColor")]
		public Color InformationColor { get; set; }

        [Editor("LogOptions.WarningColor")]
		public Color WarningColor { get; set; }

        [Editor("LogOptions.ImportantColor")]
		public Color ImportantColor { get; set; }

        [Editor("LogOptions.SuccessColor")]
		public Color SuccessColor { get; set; }

        [Editor("LogOptions.ErrorColor")]
		public Color ErrorColor { get; set; }

        [Editor("LogOptions.VerboseColor")]
		public Color VerboseColor { get; set; }

        [Editor("LogOptions.BackgroundColor")]
		public Color BackgroundColor { get; set; }

        [Editor("LogOptions.MultilineBackgroundColor")]
		public Color MultilineBackgroundColor { get; set; }

        [Editor("LogOptions.AllowResizeColumns")]
		public bool AllowResizeColumns { get; set; }

        [Editor("LogOptions.AllowRegroupColumns")]
		public bool AllowRegroupColumns { get; set; }

        [Editor("LogOptions.LogColumns", Editor = PropertyEditor.FlagsEditor, ColSpan = true, Height = 60)]
		public LogColumns LogColumns { get; set; }
		#endregion
	}
}
