﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using WideStream.Startup.Resources;

namespace WideStream.Logging
{
    internal sealed class SaveHelper
    {
        #region Construction
        private FileInfo file;
        private IEnumerable<DataGridViewRow> rows;
        private bool lastLong;

        public SaveHelper(FileInfo file, IEnumerable<DataGridViewRow> rows)
        {
            this.file = file;
            this.rows = rows;
        }
        #endregion


        #region Methods
        public void Save(LogSaveType type)
        {
            WriteCallBack callBack = null;

            switch (type)
            {
                case LogSaveType.CommaDelimited:
                    callBack = WriteCsv;
                    break;
                case LogSaveType.Html:
                    callBack = WriteHtml;
                    break;
                default:
                    callBack = WriteTab;
                    break;                
            }

            SaveText(callBack);
        }


        private void SaveText(WriteCallBack callBack)
        {
            StringBuilder sb = new StringBuilder();
            callBack(null, sb, false);

            using (StreamWriter sw = new StreamWriter(file.Create()))
            {
                foreach (DataGridViewRow tr in rows)
                    callBack(tr, sb, false);

                callBack(null, sb, true);
                sw.Write(sb.ToString());
            }
        }


        private void WriteCsv(DataGridViewRow tr, StringBuilder sb, bool finish)
        {
            if (tr != null)
                sb.AppendFormat("\"{0}\";\"{1}\";\"{2}\";\"{3}\"\r\n",
                    tr.Cells[0].Value as String,
                    tr.Cells[1].Value as String,
                    tr.Cells[2].Value as String,
                    (tr.Cells[3].Value as String).TrimStart());
        }


        private void WriteTab(DataGridViewRow tr, StringBuilder sb, bool finish)
        {
            if (tr != null)
                sb.AppendFormat("{0}\t{1}\t{2}\t{3}\r\n",
                    tr.Cells[0].Value as String,
                    tr.Cells[1].Value as String,
                    tr.Cells[2].Value as String,
                    (tr.Cells[3].Value as String).TrimStart());
        }


        private void WriteHtml(DataGridViewRow tr, StringBuilder sb, bool finish)
        {
            if (tr == null && !finish)
                sb.AppendFormat("<html><body><table border=1><tr><th>{0}</th><th>{1}</th><th>{2}</th><th>{3}</th></tr>",
                    StringTable.GetString("Log.Number"),
                    StringTable.GetString("Log.Date"),
                    StringTable.GetString("Log.Time"),
                    StringTable.GetString("Log.Message"));
            else if (tr == null && finish)
                sb.Append("</table></body></html>");
            else
            {
                string num = tr.Cells[0].Value as String;
                string msg = (tr.Cells[3].Value as String).TrimStart();

                if (String.IsNullOrEmpty(num) && !lastLong)
                {
                    lastLong = true;
                    sb.AppendFormat("<td colspan=3>&nbsp;</td><td>{0}<br>\r\n", msg);
                }
                else if (String.IsNullOrEmpty(num) && lastLong)
                    sb.AppendFormat("{0}<br>", msg);
                else
                {
                    if (lastLong)
                        sb.Append("</td></tr>");

                    sb.AppendFormat("<tr><td>{0}</td><td>{1}</td><td>{2}</td><td>{3}</td></tr>\r\n",
                    num,
                    tr.Cells[1].Value as String,
                    tr.Cells[2].Value as String,
                    msg);
                    lastLong = false;
                }                
            }
        }
        #endregion
    }
}
