﻿using System;
using System.Collections.Generic;
using System.Text;

namespace WideStream.Startup.CommandLine
{
	/// <summary>
	/// Represents a structure that contains information about an error that occured 
	/// during command line parsing (by <see cref="CommandLineParser"/> class).
	/// </summary>
	public class SwitchError
	{
		#region Construction
		private const string ERROR_FORMAT = "Type={0};Source={1}";

		/// <summary>
		/// Initializes a new instance of a structure with information about source,
		/// position and type of an error.
		/// </summary>
		/// <param name="source">A fragment of a command line that contains an error.</param>
		/// <param name="errorType">A <see cref="SwitchErrorType">type</see> of an error.</param>
		internal SwitchError(string source, SwitchErrorType errorType)
		{
			Source = source;
			ErrorType = errorType;
		}
		#endregion


		#region Methods
		/// <summary>
		/// Returns a string representation of a structure.
		/// </summary>
		/// <returns>A string in the format <i>Type=[];Source=[]</i>.</returns>
		public override string ToString()
		{
			return String.Format(ERROR_FORMAT, ErrorType.ToString(), Source);
		}
		#endregion


		#region Properties
		/// <summary>
		/// Gets a fragment of a command linethat contains an error.
		/// </summary>
		public string Source { get; private set; }
		
		/// <summary>
		/// Gets a type of an error in the fragmentwhich can be obtained 
		/// from the <see cref="Source"/> property.
		/// </summary>
		/// <value>A <see cref="SwitchErrorType">type</see> of an error.</value>
		public SwitchErrorType ErrorType { get; private set; }
		#endregion
	}
}