﻿using System;

namespace WideStream.Startup.CommandLine
{
	/// <summary>
	/// Specifies a type of a command line
	/// parser error.
	/// </summary>
	public enum SwitchErrorType
	{
		/// <summary>
		/// An error in syntax of a switch.
		/// </summary>
		SyntaxError,

		/// <summary>
		/// Switch syntax is OK but a switch
		/// wasn't recognized as a known switche.
		/// </summary>
		UnrecognizedSwitch,

		/// <summary>
		/// A syntax which is acceptable for switches 
		/// of <b>Boolean</b> type (when a value 
		/// section is omitted) was used for switch 
		/// of another type.
		/// </summary>
		MissingValue,

		/// <summary>
		/// A type of an argument specified for the command line
		/// switch is invalid (doesn't match with the expected type).
		/// </summary>
		InvalidArgumentType,

		/// <summary>
		/// No default switch is specified and a value is a command line
		/// is provided with a key.
		/// </summary>
		MissingSwitch
	}
}