﻿using System;
using System.Net.Mail;
using System.Net;

namespace WideStream.Startup.ExceptionHandling
{
    public sealed class DumpSender
    {
        #region Construction
        private const string FROM = "user@widestream.sf.net";
        private const string TITLE = "WideStream Error Report";
        private const string DEFAULT_HOST = "smtp.gmail.com";
        private const int DEFAULT_PORT = 25;
        
        public DumpSender()
        {
            From = FROM;
            Title = TITLE;
            Port = DEFAULT_PORT;
            SmtpHost = DEFAULT_HOST;
            EnableSsl = true;            
        }
        #endregion


        #region Methods
        public bool SendDump(string dump)
        {
            var eml = new MailMessage(From, To, Title, dump);
            var client = new SmtpClient(SmtpHost, Port);
            client.EnableSsl = EnableSsl;
            client.Credentials = Credentials;
          
            try
            {
                Error = null;
                client.Send(eml);
                return true;
            }
            catch (ArgumentException ex)
            {
                return ProcessException(ex);
            }
            catch (InvalidOperationException ex)
            {
                return ProcessException(ex);
            }
            catch (SmtpException ex)
            {
                return ProcessException(ex);
            }
        }


        private bool ProcessException(Exception ex)
        {
            Error = ex.Message;
            return false;
        }
        #endregion


        #region Properties
        public string From { get; set; }

        public string To { get; set; }

        public string Title { get; set; }

        public string SmtpHost { get; set; }

        public NetworkCredential Credentials { get; set; }

        public int Port { get; set; }

        public bool EnableSsl { get; set; }

        public string Error { get; private set; }
        #endregion
    }
}
