﻿using System;
using System.Text;
using System.Windows.Forms;

namespace WideStream.Startup.ExceptionHandling
{
    public sealed class ExceptionDump
    {
        #region Construction
        private const string HDR_DATETIME = "DateTime: {0}\r\n";
        private const string HDR_URL = "Url: {0}\r\n";
        private const string HDR_WIDESTREAM = "WideStream: {0}\r\n";
        private const string HDR_OS = "Windows: {0}\r\n";
        private const string HDR_WORKINGSET = "WorkingSet: {0}\r\n";
        private const string HDR_CMDLINE = "CommandLine: {0}\r\n";
        private const string HDR_TICKS = "Ticks: {0}\r\n";
        private const string HDR_RUNTIME = "Runtime: {0}\r\n";
        private const string HDR_DATA = "Data:\r\n {0}\r\n";
        
        public ExceptionDump(Exception ex, Uri url)
        {
            Exception = ex;
            Url = url;
        }
        #endregion


        #region Methods
        public string GenerateDump(bool detailed)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendFormat(HDR_DATETIME, DateTime.Now);
            
            if (Url != null)
                sb.AppendFormat(HDR_URL, Url);

            if (detailed)
            {
                sb.AppendFormat(HDR_WIDESTREAM, Application.ProductVersion);
                sb.AppendFormat(HDR_OS, Environment.OSVersion);
                sb.AppendFormat(HDR_RUNTIME, Environment.Version);
                sb.AppendFormat(HDR_WORKINGSET, Environment.WorkingSet);
                sb.AppendFormat(HDR_TICKS, Environment.TickCount);
                sb.AppendFormat(HDR_CMDLINE, Environment.CommandLine);
            }

            if (Exception != null)
                sb.AppendFormat(HDR_DATA, Exception.ToString());

            return sb.ToString();
        }
        #endregion


        #region Properties
        public Exception Exception { get; private set; }

        public Uri Url { get; private set; }
        #endregion
    }
}
