﻿using System;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using System.Net.Mail;
using WideStream.Startup.ExceptionHandling;
using System.Net;

namespace WideStream.Startup.ExceptionHandling
{
	public static class ExceptionManager
	{
		#region Methods
        private static object syncRoot = new Object();

		public static void Log(Exception ex)
		{
			Log(ex, null);
		}


        public static void Process(Exception exception, Uri url)
		{
			if (exception is ThreadAbortException)
				return;

            lock (syncRoot)
            {
                ExceptionDialog d = new ExceptionDialog();
                d.ExceptionMessage = exception.Message;
                d.ExceptionDetails = exception.ToString();
                d.ShowDialog(Program.Environment.MainForm);
                var dump = new ExceptionDump(exception, url);
                
                if (d.SendMail)
                {
                    var sender = new DumpSender
                    {
                        To = SenderData.Recipient,
                        Credentials = new NetworkCredential(SenderData.Login, SenderData.Password)
                    };

                    if (!sender.SendDump(dump.GenerateDump(true)))
                        MessageBox.Show(sender.Error, Application.ProductName, 
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                Log(exception, url);
                Application.Exit();
            }
		}



        public static void Log(Exception exception, Uri url)
        {
            if (ErrorLogPath != null)
            {
                try
                {
                    var dump = new ExceptionDump(exception, url);
                    
                    using (StreamWriter writer = new StreamWriter(
                        File.Open(ErrorLogPath.FullName, FileMode.Append)))
                        writer.Write(dump.GenerateDump(false));
                }
                catch (Exception ex)
                {
                    if (!(ex is ArgumentException) ||
                        !(ex is IOException) ||
                        !(ex is UnauthorizedAccessException) ||
                        !(ex is NotSupportedException))
                        throw;
                }
            }
        }
		#endregion


		#region Properties
		public static FileInfo ErrorLogPath { get; set; }
		#endregion
	}
}
