﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;
using WideStream.Integration;
using System.IO;
using System.Drawing;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.DownloadManager.Network;
using WideStream.Configuration.ComponentModel;
using WideStream.Configuration;
using WideStream.Startup.Resources;

namespace WideStream.Startup
{
    public sealed class ManifestReader
    {
        public void Read(XmlReader reader)
        {
            while (reader.Read())
            {
                if (String.Compare(reader.Name, "Integrations", true) == 0 &&
                    reader.IsStartElement())
                    ReadIntegrations(reader.ReadSubtree());
                else if (String.Compare(reader.Name, "Protocols", true) == 0 &&
                    reader.IsStartElement())
                    ReadProtocols(reader.ReadSubtree());
                else if (String.Compare(reader.Name, "Adapters", true) == 0 &&
                    reader.IsStartElement())
                    ReadAdapters(reader.ReadSubtree());
            }
        }


        private void ReadIntegrations(XmlReader reader)
        {
            while (reader.ReadToFollowing("Integration"))
            {
                Type type = Type.GetType(reader.GetAttribute("Type"));
                IntegrationAddin addin = new IntegrationAddin(
                    reader.GetAttribute("BrowserName"),
                    Image.FromStream(GetResource(type, reader.GetAttribute("BrowserImage"))),
                    GetDescription(reader),
                    (IBrowserIntegration)Activator.CreateInstance(type));
                IntegrationManager.RegisterIntegration(addin);
            }
        }


        private void ReadProtocols(XmlReader reader)
        {
            while (reader.ReadToFollowing("Protocol"))
            {
                Type type = Type.GetType(reader.GetAttribute("Options"));
                Config.Register(type);
                Options opt = Config.Query(type.GUID);
                DownloadProtocol dt = new DownloadProtocol(
                    reader.GetAttribute("Key"), reader.GetAttribute("Name"),
                    (IDownloadProvider)Activator.CreateInstance(Type.GetType(reader.GetAttribute("Type"))), 
                    opt) 
                    {  Description = GetDescription(reader) };
                DownloadProtocolManager.RegisterProtocol(dt);
               
            }
        }


        private void ReadAdapters(XmlReader reader)
        {
            while (reader.ReadToFollowing("Adapter"))
            {
                Type type;
                DownloadAdapterAddin add = new DownloadAdapterAddin(
                    reader.GetAttribute("Key"),
                    reader.GetAttribute("Name"),
                    reader.GetAttribute("MatchPattern"),
                    GetDescription(reader),
                    type = Type.GetType(reader.GetAttribute("Type")),
                    Image.FromStream(GetResource(type, reader.GetAttribute("Image"))));
                DownloadAdapterManager.RegisterAdapter(add);
            }
        }


        private string GetDescription(XmlReader reader)
        {
            string desc = reader.GetAttribute("Description");

            if (String.IsNullOrEmpty(desc))
                return desc;

            if (desc[0] == '$')
                desc = StringTable.GetString(desc);
            
            desc = desc.Replace("%n%", Environment.NewLine);
            return desc;
        }


        private Stream GetResource(Type type, string resource)
        {
            return type.Assembly.GetManifestResourceStream(resource);
        }
    }
}
