﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using WideStream.Workspace;
using WideStream.Startup.Activation;
using WideStream.Startup.CommandLine;
using WideStream.Startup.ExceptionHandling;

namespace WideStream.Startup
{
    public static class Program
	{
		#region Construction
		private const int EC_TERMIN = 1;
		private const int EC_NORMAL = 0;
		#endregion


		#region Methods
		/// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        private static int Main(params string[] args)
        {
			CommandLineProperties startupProperties = ProcessCommandLine(args);
			
			if (startupProperties == null)
				return EC_TERMIN;

			using (IProgramChannel client = ExecutionChannel.CreateChannel<ClientChannel>())
			{
				IActivationService activated = client.DiscoverService();
				
				if (activated != null)
				{
					if (startupProperties.Url != null)
						activated.AddUrl(startupProperties.Url);
					else
						activated.Activate();

					return EC_TERMIN;
				}
			}
						
			using (IProgramChannel service = ExecutionChannel.CreateChannel<ServiceChannel>())
			{
				Application.EnableVisualStyles();
				Application.SetCompatibleTextRenderingDefault(false);
				Main form = new Main();
				Environment = new ApplicationEnvironment(form, startupProperties);
				
				if (startupProperties.Url != null)
					new ActivationService().AddUrl(startupProperties.Url);

                Application.Run(Environment);
				return EC_NORMAL;
			}		
		}


		private static CommandLineProperties ProcessCommandLine(params string[] args)
		{
			CommandLineParser parser = new CommandLineParser();
			CommandLineProperties properties = parser.Parse<CommandLineProperties>(args);

			if (parser.Errors.Count > 0)
			{
				MessageBox.Show(parser.Errors.ToString(), Application.ProductName,
					MessageBoxButtons.OK, MessageBoxIcon.Error);
				return null;
			}
			else
			{
				if (!parser.FoundSwitches && !String.IsNullOrEmpty(properties.Url))
					properties.Minimize = true;

				return properties;
			}
		}
		#endregion


		#region Properties
		public static ApplicationEnvironment Environment { get; private set; }
		#endregion
	}
}
