﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Windows.Forms;
using WideStream.Startup.Resources;

namespace WideStream.Workspace
{
    partial class AboutDialog : Form
    {
        #region Construction
        private Image logo;

        public AboutDialog()
        {
            InitializeComponent();
            Text = StringTable.GetString(Text);
            closeButton.Text = StringTable.GetString(closeButton.Text);
            tabPage1.Text = StringTable.GetString(tabPage1.Text);
            tabPage2.Text = StringTable.GetString(tabPage2.Text);
            Text = String.Format(Text, AssemblyProduct);
            titleLabel.Text = String.Format(titleLabel.Text, AssemblyVersion, AssemblyTitle, AssemblyConfiguration);
            copyrightLabel.Text = AssemblyCopyright;
            descriptionTextBox.Text = AssemblyMilestoneDescription;
            copyrightTextBox.Text = AssemblyLicense;
        }
        #endregion

        
        #region Methods
        private void AboutDialog_Load(object sender, EventArgs e)
        {
            headerPanel.Paint += delegate(object send, PaintEventArgs ev)
            {
                ev.Graphics.DrawLine(Pens.DarkGray, 0, headerPanel.ClientSize.Height - 1, 
                    headerPanel.ClientSize.Width, headerPanel.ClientSize.Height - 1);
            };
        }


        protected override void OnClosed(EventArgs e)
        {
            if (logo != null)
            {
                logo.Dispose();
                logo = null;
            }

            base.OnClosed(e);
        }
        #endregion


        #region Properties
        public string AssemblyTitle
        {
            get
            {
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyTitleAttribute), false);
                if (attributes.Length > 0)
                {
                    AssemblyTitleAttribute titleAttribute = (AssemblyTitleAttribute)attributes[0];
                    if (titleAttribute.Title != String.Empty)
                    {
                        return titleAttribute.Title;
                    }
                }
                return System.IO.Path.GetFileNameWithoutExtension(Assembly.GetExecutingAssembly().CodeBase);
            }
        }


        public string AssemblyVersion
        {
            get
            {
                return Assembly.GetExecutingAssembly().GetName().Version.ToString();
            }
        }


        public string AssemblyLicense
        {
            get
            {
                return StringTable.GetString("General.ProductLicense");
            }
        }


        public string AssemblyMilestoneDescription
        {
            get
            {
                return StringTable.GetString("General.ProductMilestoneDescription");
            }
        }



        public string AssemblyProduct
        {
            get
            {
                return StringTable.GetString("General.ProductNameShort");
            }
        }


        public string AssemblyCopyright
        {
            get
            {
                return StringTable.GetString("General.ProductCopyright");
            }
        }


        public string AssemblyDeveloper
        {
            get
            {
                return StringTable.GetString("General.ProductDeveloper");
            }
        }


        public string AssemblyConfiguration
        {
            get
            {
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyConfigurationAttribute), false);
                if (attributes.Length == 0)
                {
                    return String.Empty;
                }
                return ((AssemblyConfigurationAttribute)attributes[0]).Configuration;
            }
        }
        #endregion
    }
}
