﻿using System;
using WideStream.Configuration.ComponentModel;

namespace WideStream.Workspace.Configuration
{
    [Options(Category = "WorkspaceOptions.Category", LongName = "WorkspaceOptions.LongName", ShortName = "WorkspaceOptions.ShortName")]
	public sealed class WorkspaceOptions : Options
	{
		#region Construction
		public WorkspaceOptions()
		{
			ToolbarPosition = MenuPosition = StripPosition.Floating;
			Theme = VisualTheme.ApplicationControlled;
			PromptDialogs = PromptDialogs.RemoveDownload | PromptDialogs.AbortDownload;
			MinimizedState = MinimizedState.ShowIconHide;
			ShowSaveProgressBar = LogVisible = ToolbarVisible = StatusBarVisible = true;
		}
		#endregion


		#region Properties
		public bool LogVisible { get; set; }

		public bool ToolbarVisible { get; set; }

		public bool StatusBarVisible { get; set; }

        [Editor("WorkspaceOptions.Language", Editor = PropertyEditor.LanguageComboBox, RequiresRestart = true)]
		public string Language { get; set; }

        [Editor("WorkspaceOptions.ToolbarPosition")]
		public StripPosition ToolbarPosition { get; set; }

        [Editor("WorkspaceOptions.MenuPosition")]
		public StripPosition MenuPosition { get; set; }

        [Editor("WorkspaceOptions.Theme", RequiresRestart = true)]
		public VisualTheme Theme { get; set; }

        [Editor("WorkspaceOptions.MinimizedState")]		
		public MinimizedState MinimizedState { get; set; }

        [Editor("WorkspaceOptions.ShowSaveProgressBar")]
		public bool ShowSaveProgressBar { get; set; }

        [Editor("WorkspaceOptions.PromptDialogs", Editor = PropertyEditor.FlagsEditor, ColSpan = true, Height = 60)]		
		public PromptDialogs PromptDialogs { get; set; }        
		#endregion
	}
}
