﻿using System;
using System.Windows.Forms;

namespace WideStream.Workspace.Forms
{
    public sealed class BufferedDataGridView : DataGridView
	{
		#region Construction
		private bool enterDown;

		public BufferedDataGridView()
        {
            SetStyle(ControlStyles.UserPaint | ControlStyles.ResizeRedraw | 
				ControlStyles.OptimizedDoubleBuffer | ControlStyles.AllPaintingInWmPaint, true);
		}
		#endregion
		

		#region Methods
		protected override void OnKeyDown(KeyEventArgs e)
		{
			if (e.KeyData == Keys.Enter)
				e.Handled = (enterDown = true);

			base.OnKeyDown(e);
		}


		protected override void OnKeyUp(KeyEventArgs e)
		{
			if (e.KeyData == Keys.Enter && enterDown)
			{
				enterDown = false;
				OnEnterPressed();
			}

			base.OnKeyUp(e);
		}
		#endregion


		#region Events
		public event EventHandler EnterPressed;
		private void OnEnterPressed()
		{
			EventHandler h = EnterPressed;

			if (h != null)
				h(this, EventArgs.Empty);
		}
		#endregion
	}
}
