﻿using System;
using System.Drawing;
using System.Reflection;
using System.Windows.Forms;
using WideStream.Configuration.ComponentModel;
using WideStream.Startup.Resources;

namespace WideStream.Workspace.Forms
{
	public sealed class ColorPicker : ComboBox, IEditor
	{
		#region Construction
		private const string COLOR_STRING = "Red,Green,Blue,Orange,White,Black,Yellow,Brown,Transparent,DimGray,Gray,DarkGray,Silver,LightGray,Gainsboro,WhiteSmoke,RosyBrown,IndianRed,FireBrick,Maroon,DarkRed,Snow,MistyRose,LightCoral,Salmon,Tomato,LightSalmon,DarkSalmon,Coral,OrangeRed,Sienna,SeaShell,Chocolate,SaddleBrown,SandyBrown,PeachPuff,Peru,Linen,Bisque,DarkOrange,BurlyWood,Tan,AntiqueWhite,NavajoWhite,Wheat,PapayaWheap,DarkGoldenrod,Goldenrod,PaleGoldenrod,Gold,LightGoldenrodYellow,LemonChiffon,Khaki,DarkKhaki,LightYellow,Olive,Beige,YellowGreen,DarkSeaGreen,ForestGreen,DarkGreen,Lime,SpringGreen,DarkSlateGray,DeepSkyBlue,LightBlue,LightSkyBlue,SteelBlue,DodgerBlue,CornflowerBlue,MidnightBlue,DarkBlue,Navy,BlueViolet,Indigo,Lavender,DarkViolet,DarkOrchid,MediumOrchid,Orchid,Violet,Purple,Pink,LightPink,HotPink";
		private const string SYSCOLOR_STRING = "ActiveBorder,ActiveCaption,ActiveCaptionText,AppWorkspace,Control,ControlDark,ControlDarkDark,ControlLight,ControlLightLight,ControlText,Desktop,Highlight,HighlightText,GrayText,Menu,MenuText,Window,WindowFrame,WindowText,InactiveBorder,InactiveCaption,InactiveCaptionText,Info,InfoText,ScrollBar";

		public ColorPicker()
		{
			Items.AddRange(COLOR_STRING.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries));
			Items.AddRange(SYSCOLOR_STRING.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries));
			SelectedIndex = 0;
			DropDownStyle = ComboBoxStyle.DropDownList;
			DrawMode = DrawMode.OwnerDrawVariable;
			SetStyle(ControlStyles.ResizeRedraw | ControlStyles.AllPaintingInWmPaint |
				ControlStyles.OptimizedDoubleBuffer, true);
		}
		#endregion


		#region Methods
		protected override void OnMeasureItem(System.Windows.Forms.MeasureItemEventArgs e)
		{
			e.ItemHeight = 16;
		}


		protected override void OnDrawItem(System.Windows.Forms.DrawItemEventArgs e)
		{
			if (e.Index == -1)
				return;

			Rectangle fullRect = new Rectangle(e.Bounds.X + 1, e.Bounds.Y,
				e.Bounds.Width - 4, e.Bounds.Height - 1);
			Rectangle colorRect = new Rectangle(e.Bounds.X + 2, e.Bounds.Y + 2, 12, 10);
			Brush textBrush = SystemBrushes.ControlText;

			if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
			{
				e.Graphics.FillRectangle(SystemBrushes.Highlight, fullRect);
				textBrush = SystemBrushes.HighlightText;
			}
			else
				e.Graphics.FillRectangle(SystemBrushes.Window, e.Bounds);

			string itemValue = this.Items[e.Index].ToString();

			using (Brush colorBrush = new SolidBrush(Color.FromName(itemValue)))
			{
				itemValue = StringTable.GetString("ColorPicker." + itemValue);
				e.Graphics.FillRectangle(colorBrush, colorRect);
				e.Graphics.DrawRectangle(Pens.Black, colorRect);
				e.Graphics.DrawString(itemValue, Font, textBrush,
					new Point(e.Bounds.X + 18, e.Bounds.Y + 1));
			}
		}
		
		
		protected override void OnSelectedItemChanged(EventArgs e)
		{
            IsDirty = true;
			OnValueChanged(new EditorEventArgs(Value));
			base.OnSelectedItemChanged(e);
		}


		public void SetStandardValues(object value)
		{
	
		}
		#endregion
		

		#region Properties
		public Color SelectedColor 
		{
			get
			{
				if (SelectedItem != null)
					return Color.FromName(SelectedItem.ToString());

				return default(Color);
			}
		}


		public object Value
		{
			get { return SelectedColor; }
			set { Text = ((Color)value).Name; }
		}


		public PropertyInfo RelatedProperty { get; set; }
		
        public bool IsDirty { get; private set; }
        #endregion
		
		
		#region Events
		public event EventHandler<EditorEventArgs> ValueChanged;
		private void OnValueChanged(EditorEventArgs e)
		{
			EventHandler<EditorEventArgs> h = ValueChanged;
			
			if (h != null)
				h(this, e);
		}
		#endregion
	}
}