﻿using System;
using System.Reflection;
using System.Windows.Forms;
using WideStream.Configuration.ComponentModel;
using WideStream.Core.Network;
using WideStream.Startup.Resources;

namespace WideStream.Workspace.Forms
{
	public sealed class CredentialsEditor : Panel, IEditor
	{
		#region Construction
		private CheckBox checkBox;
		private TextBox userName;
		private TextBox password;

		public CredentialsEditor()
		{
			Initialize();
			base.Padding = new Padding(0, 12, 0, 0);
		}
		#endregion


		#region Methods
		private void Initialize()
		{
			TableLayoutPanel panel = new TableLayoutPanel();
			panel.Dock = DockStyle.Fill;
			panel.RowCount = 3;
			panel.ColumnCount = 3;
			panel.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 50F));
			panel.ColumnStyles.Add(new ColumnStyle(SizeType.Absolute, 10F));
			panel.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 50F));

			checkBox = new CheckBox();
			checkBox.Text = StringTable.GetString("Credentials.Credentials");
			checkBox.AutoSize = true;
			checkBox.TabIndex = TabIndex;
			checkBox.CheckedChanged += (o, e) => userName.Enabled = password.Enabled = checkBox.Checked;
			panel.Controls.Add(checkBox, 0, 0);
			panel.SetColumnSpan(checkBox, 3);

            panel.Controls.Add(CreateLabel(StringTable.GetString("Credentials.UserName"), TabIndex + 1), 0, 1);
			panel.Controls.Add(userName = CreateTextBox(false, TabIndex + 2), 0, 2);
            panel.Controls.Add(CreateLabel(StringTable.GetString("Credentials.Password"), TabIndex + 3), 3, 1);
			panel.Controls.Add(password = CreateTextBox(true, TabIndex + 4), 3, 2);
			Controls.Add(panel);
		}


		private Label CreateLabel(string caption, int tabIndex)
		{
			Label label = new Label();
			label.AutoSize = true;
			label.Margin = new Padding(0, 12, 3, 0);
			label.Text = caption;
			label.TabIndex = tabIndex;
			return label;
		}


		private TextBox CreateTextBox(bool password, int tabIndex)
		{
			TextBox tb = new TextBox();
			tb.UseSystemPasswordChar = password;
			tb.TabIndex = TabIndex;
			tb.Dock = DockStyle.Fill;
			tb.Enabled = false;
            tb.TextChanged += (o, e) => IsDirty = true;
			return tb;
		}


		public void  SetStandardValues(object value)
		{

		}
		#endregion

	
		#region Properties
		public object Value
		{
			get 
			{ 
				return !checkBox.Checked || String.IsNullOrEmpty(userName.Text) ||
					String.IsNullOrEmpty(password.Text) ? null :
					new	Credentials(userName.Text, password.Text);
			}
			set 
			{
				checkBox.Checked = value != null && value is Credentials &&
					!String.IsNullOrEmpty(((Credentials)value).UserName);
				
				if (checkBox.Checked)
				{
					userName.Text = ((Credentials)value).FullUserName;
					password.Text = ((Credentials)value).Password;
				}
			}
		}


        public Credentials Credentials
        {
            get { return Value as Credentials; }
            set { Value = value; }
        }


		public PropertyInfo RelatedProperty { get; set; }

        public bool IsDirty { get; private set; }
        #endregion


		#region Events
		public event EventHandler<EditorEventArgs>  ValueChanged;
		private void OnValueChanged(EditorEventArgs e)
		{
			EventHandler<EditorEventArgs> h = ValueChanged;

			if (h != null)
				ValueChanged(this, e);
		}


		public event EventHandler CredentialsChanged;
		private void OnCredentialsChanged()
		{
			EventHandler h = CredentialsChanged;

			if (h != null)
				h(this, EventArgs.Empty);
		}
		#endregion
	}
}
