﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Drawing;
using WideStream.DownloadManager;
using WideStream.Configuration;
using WideStream.DownloadManager.Configuration;

namespace WideStream.Workspace.Forms
{	
    public class DataGridViewIconCell : DataGridViewCell
	{
		#region Construction
		public DataGridViewIconCell()
		{
			Padding = 8;
			OverdrawImage = true;
		}
		#endregion


		#region Methods
		protected override void Paint(Graphics graphics, Rectangle clipBounds, Rectangle cellBounds,
            int rowIndex, DataGridViewElementStates cellState, object value, object formattedValue,
            string errorText, DataGridViewCellStyle cellStyle, DataGridViewAdvancedBorderStyle advancedBorderStyle,
            DataGridViewPaintParts paintParts)
        {
            using (Brush textBrush = new SolidBrush(Style.ForeColor != default(Color) ? Style.ForeColor : SystemColors.ControlText))
            {
                graphics.FillRectangle(SystemBrushes.Window, new Rectangle(cellBounds.X, cellBounds.Y,
                    cellBounds.Width, cellBounds.Height));
                Brush highBrush = null;

                if ((cellState & DataGridViewElementStates.Selected) == DataGridViewElementStates.Selected)
                {
                    graphics.FillRectangle(SystemBrushes.Highlight, OverdrawImage ? cellBounds :
                        new Rectangle(cellBounds.X + Padding + 18, cellBounds.Y, cellBounds.Width, cellBounds.Height));
                    highBrush = SystemBrushes.HighlightText;
                }

				DownloadManagerDisplayOptions conf = Config.Query<DownloadManagerDisplayOptions>();
                bool displayIcon = Image != null && conf.DisplayPriorityIcon ||
                    Image == null && conf.DisplaySpeedChangeIcon;
                int shift = 0;

                if (Image != null)
                {
                    graphics.DrawImage(Image, cellBounds.X + Padding + shift, cellBounds.Y);
                    shift += 20;
                }
				
                if (SmallImage != null && displayIcon)
                {
                    shift -= 4;
                    graphics.DrawImage(SmallImage, cellBounds.X + Padding + shift, cellBounds.Y + 6);
                    shift += 8;
                }

				string drawValue = TextCallBack != null ? TextCallBack() : value as String;

				graphics.DrawString(drawValue, InheritedStyle.Font, highBrush ?? textBrush,
                    new PointF(cellBounds.X + shift + Padding, cellBounds.Y + 2));

                if (base.OwningRow.DataGridView.CellBorderStyle != DataGridViewCellBorderStyle.None)
                    using (Pen p = new Pen(base.OwningRow.DataGridView.GridColor))
                        graphics.DrawRectangle(p, cellBounds.X - 1, cellBounds.Y - 1,
                            cellBounds.Width, cellBounds.Height);
            }
		}
		#endregion


		#region Properties
		public Image Image { get; set; }

		public Image SmallImage { get; set; }

        public int Padding { get; set; }
		
		public VariableTextCallBack TextCallBack { get; set; }

		public bool OverdrawImage { get; set; }

        public override Type FormattedValueType
        {
            get { return typeof(String); }
		}
		#endregion
	}
}
