﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Drawing;

namespace WideStream.Workspace.Forms
{
    public class DataGridViewProgressBarCell : DataGridViewCell
    {
        protected override void Paint(Graphics graphics, Rectangle clipBounds, Rectangle cellBounds, 
            int rowIndex, DataGridViewElementStates cellState, object value, object formattedValue, 
            string errorText, DataGridViewCellStyle cellStyle, DataGridViewAdvancedBorderStyle advancedBorderStyle, 
            DataGridViewPaintParts paintParts)
        {
            if (value == null)
                value = 0;

            Brush textBrush = SystemBrushes.ControlText;
            Pen progressPen = Pens.Navy;
            Brush progressBrush = Brushes.Navy;

            if ((cellState & DataGridViewElementStates.Selected) == DataGridViewElementStates.Selected)
            {
                graphics.FillRectangle(SystemBrushes.Highlight, cellBounds);
                textBrush = SystemBrushes.HighlightText;
                progressPen = SystemPens.HighlightText;
                progressBrush = SystemBrushes.HighlightText;
            }
            else
                graphics.FillRectangle(SystemBrushes.Window, new Rectangle(cellBounds.X, cellBounds.Y,
                    cellBounds.Width, cellBounds.Height));

            if (Convert.ToInt64(value) < 0)
                value = 0;

            string percent = String.Format("{0}%", value.ToString());
            SizeF size = graphics.MeasureString("100%", SystemFonts.MenuFont);
            graphics.DrawRectangle(progressPen, new Rectangle(cellBounds.X + 4, cellBounds.Y + 3,
                cellBounds.Width - 8 - (Int32)size.Width, cellBounds.Height - 6));
            graphics.FillRectangle(progressBrush, new RectangleF(cellBounds.X + 4, cellBounds.Y + 3,
                (Single)(cellBounds.Width - 8 - size.Width) / 100F * Convert.ToInt64(value), cellBounds.Height - 6));
            graphics.DrawString(percent, this.InheritedStyle.Font, textBrush,
                new PointF(cellBounds.X + cellBounds.Width - size.Width, cellBounds.Y + 2));
           
            if (base.OwningRow.DataGridView.CellBorderStyle != DataGridViewCellBorderStyle.None)
                using (Pen p = new Pen(base.OwningRow.DataGridView.GridColor))
                    graphics.DrawRectangle(p, cellBounds.X - 1, cellBounds.Y - 1, 
                        cellBounds.Width, cellBounds.Height);
        }



        public override Type FormattedValueType
        {
            get { return typeof(String); }
        }
    }
}
