﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using WideStream.Configuration.ComponentModel;
using System.Reflection;
using WideStream.Core.Runtime;
using WideStream.Startup.Resources;

namespace WideStream.Workspace.Forms
{
	public sealed class DropDownList : ComboBox, IEditor
	{
		#region Construction
		public DropDownList()
		{
			DropDownStyle = ComboBoxStyle.DropDownList;
			ValueMember = "Value";
			DisplayMember = "Name";
		}
		#endregion


		#region Methods
		public void SetStandardValues(object value)
		{
			Items.Clear();
			Type type = value.GetType();

			if (type.IsSubclassOf(typeof(Enum)))
			{
				foreach (Enum v in Enum.GetValues(type))
					Items.Add(new ValuePair<String,Int32>(v.ToDisplayString(), v.DirectCast<Int32>()));

				SelectedValue = value.DirectCast<Int32>();
			}
			else if (type == typeof(Boolean))
			{
				Items.Add(new ValuePair<String,Int32>(StringTable.GetString("DropDownList.Yes"), 1));
				Items.Add(new ValuePair<String,Int32>(StringTable.GetString("DropDownList.No"), 0));
				SelectedValue = ((Boolean)value).DirectCast<Int32>();
			}
		}


        protected override void OnSelectedIndexChanged(EventArgs e)
		{
            IsDirty = true;
			OnValueChanged(new EditorEventArgs(Value));
			base.OnSelectedItemChanged(e);
		}
		#endregion


		#region Properties
		public object Value
		{
			get 
			{
				if (RelatedProperty != null && RelatedProperty.PropertyType == typeof(Boolean))
					return ((ValuePair<String,Int32>)SelectedItem).Value == 1;
				else
					return ((ValuePair<String,Int32>)SelectedItem).Value;
			}
			set
			{
				Text = value is Enum ? ((Enum)value).ToDisplayString() :
                  (((Boolean)value) ? StringTable.GetString("DropDownList.Yes") : StringTable.GetString("DropDownList.No"));
                IsDirty = false;
			}
		}

		public PropertyInfo RelatedProperty { get; set; }

        public bool IsDirty { get; private set; }
        #endregion
		
				
		#region Events
		public event EventHandler<EditorEventArgs> ValueChanged;
		private void OnValueChanged(EditorEventArgs e)
		{
			EventHandler<EditorEventArgs> h = ValueChanged;
			
			if (h != null)
				h(this, e);
		}
		#endregion
	}
}
