﻿using System;
using System.IO;
using System.Reflection;
using System.Windows.Forms;
using WideStream.Configuration.ComponentModel;
using System.Drawing;

namespace WideStream.Workspace.Forms
{
	public abstract class FileSystemPicker : Panel, IEditor
	{   
		#region Construction
		private TextEditor textBox;
		private Button button;

        public FileSystemPicker()
		{
			InitializeControls();
		}
		#endregion


		#region Methods
		public void InitializeControls()
		{
			TableLayoutPanel panel = new TableLayoutPanel();
			panel.ColumnCount = 2;
			panel.RowCount = 1;
			panel.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 100));
			panel.ColumnStyles.Add(new ColumnStyle(SizeType.Absolute, 30));
			panel.Dock = DockStyle.Fill;
			Controls.Add(panel);

			textBox = new TextEditor();
			textBox.Dock = DockStyle.Fill;
			textBox.Margin = new Padding(0, 0, 3, 0);
			textBox.TextChanged += (o, e) => OnValueChanged(new EditorEventArgs(Value));
            InitializeTextBox(textBox);
			panel.Controls.Add(textBox, 0, 0);

			button = new Button();
			button.Height = textBox.Height;
			button.Margin = new Padding(3, 0, 0, 0);
			button.Click += (o, e) => ButtonClicked();
			button.Text = "...";
			button.Width = 30;
			panel.Controls.Add(button, 1, 0);
		}


        protected virtual void InitializeTextBox(TextBox textBox)
        {
            
        }


        protected abstract void ButtonClicked();


		protected override void OnResize(EventArgs e)
		{
			Height = Controls[0].Controls[0].Height + 6;
			base.OnResize(e);
		}	


		public void SetStandardValues(object value)
		{

		}
		#endregion


		#region Properties
		public TextBox TextBox 
		{
			get { return textBox; }
		}


        public new bool Enabled
        {
            get { return textBox.Enabled && button.Enabled; }
            set { textBox.Enabled = button.Enabled = value; }
        }


		public object Value
		{
            get { return textBox.Text; }
            set { textBox.Text = value as String; }
		}

		public PropertyInfo RelatedProperty { get; set; }

        public bool IsDirty { get; private set; }
        #endregion
						
				
		#region Events
		public event EventHandler<EditorEventArgs> ValueChanged;
		private void OnValueChanged(EditorEventArgs e)
		{
            IsDirty = true;
            EventHandler<EditorEventArgs> h = ValueChanged;
			
			if (h != null)
				h(this, e);
		}
		#endregion
	}
}
