﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WideStream.Configuration.ComponentModel;
using System.Windows.Forms;
using System.Reflection;
using WideStream.Core.Runtime;

namespace WideStream.Workspace.Forms
{
	public sealed class FlagsEditor : CheckedListBox, IEditor
	{
		#region Construction
		public FlagsEditor()
		{
			ValueMember = "Value";
			DisplayMember = "Name";
		}
		#endregion


		#region Methods
		public void SetStandardValues(object value)
		{
			Items.Clear();

			foreach (Enum v in Enum.GetValues(value.GetType()))
			{
				if (v.HasDisplayName())
				{
					Items.Add(new ValuePair<String,Int32>(v.ToDisplayString(),
						(Int32)Convert.ChangeType(v, typeof(Int32))));
					SetItemChecked(Items.Count - 1,
						(value.DirectCast<Int32>() & v.DirectCast<Int32>()) == v.DirectCast<Int32>());
				}
			}
		}
		
		
		protected override void OnItemCheck(ItemCheckEventArgs e)
		{
            IsDirty = true;
			OnValueChanged(new EditorEventArgs(Value));
			base.OnItemCheck(e);
		}
		#endregion


		#region Properties
		public object Value
		{
			get
			{
				int retval = 0;

				foreach (ValuePair<String,Int32> p in CheckedItems)
					retval |= p.Value;

				return retval;
			}
			set { SetStandardValues(value); }
		}

		public PropertyInfo RelatedProperty { get; set; }

        public bool IsDirty { get; private set; }
        #endregion
				
				
		#region Events
		public event EventHandler<EditorEventArgs> ValueChanged;
		private void OnValueChanged(EditorEventArgs e)
		{
			EventHandler<EditorEventArgs> h = ValueChanged;
			
			if (h != null)
				h(this, e);
		}
		#endregion
	}
}
