﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace WideStream.Workspace.Forms
{
    public sealed class GroupListBox : ListBox
    {
        #region Construction
        private bool moveUp;

        public GroupListBox()
        {
            SetStyle(ControlStyles.AllPaintingInWmPaint | ControlStyles.OptimizedDoubleBuffer, true);
            DrawMode = DrawMode.OwnerDrawFixed;
        }
        #endregion


        #region Methods
        protected override void OnSelectedIndexChanged(EventArgs e)
        {
            base.OnSelectedIndexChanged(e);

            if (SelectedIndex != -1 && Items[SelectedIndex].ToString().StartsWith(">"))
                SelectedIndex += moveUp && SelectedIndex > 1 ? -1 : 1;
        }


        protected override void OnMeasureItem(MeasureItemEventArgs e)
        {
            e.ItemWidth = ClientSize.Width;
            e.ItemHeight = 26;
            base.OnMeasureItem(e);
        }


        protected override void OnDrawItem(DrawItemEventArgs e)
        {
            if (e.Index != -1)
            {
                if (Items[e.Index].ToString().StartsWith(">"))
                    DrawHeader(Items[e.Index].ToString().Trim('>'), e);
                else
                    DrawNormalItem(Items[e.Index].ToString(), e);
            }

            base.OnDrawItem(e);
        }


        protected override void OnKeyDown(KeyEventArgs e)
        {
            moveUp = (Int32)e.KeyCode == 38;
            base.OnKeyDown(e);
        }


        private void DrawNormalItem(string caption, DrawItemEventArgs e)
        {
            caption = caption.Split('|')[0];
            Brush brush;

            if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
            {
                brush = SystemBrushes.HighlightText;
                e.Graphics.FillRectangle(SystemBrushes.Highlight, 
                    new Rectangle(e.Bounds.X, e.Bounds.Y + 0, e.Bounds.Width, e.Bounds.Height - 0));
            }
            else
            {
                brush = SystemBrushes.ControlText;
                e.Graphics.FillRectangle(SystemBrushes.Window, e.Bounds);
            }
          
            SizeF captionSize = e.Graphics.MeasureString(caption, Font);
            e.Graphics.DrawString(caption, e.Font, brush,
                e.Bounds.X + 10, e.Bounds.Y + (e.Bounds.Height - captionSize.Height) / 2);
        }


        private void DrawHeader(string caption, DrawItemEventArgs e)
        {
            e.Graphics.FillRectangle(SystemBrushes.Window, e.Bounds);
            Font font = new Font(e.Font, FontStyle.Bold);
            SizeF captionSize = e.Graphics.MeasureString(caption, font);
            e.Graphics.DrawString(caption, font, SystemBrushes.ControlText,
                e.Bounds.X + 2, e.Bounds.Y + (e.Bounds.Height - captionSize.Height) / 2);
        }


        public int GetSelectedValue()
        {
            if (SelectedIndex != -1)
            {
                string item = Items[SelectedIndex].ToString();

                if (!item.StartsWith(">"))
                    return Int32.Parse(item.Split('|')[1]);
            }

            return -1;
        }
        #endregion
    }
}
