﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using WideStream.Configuration.ComponentModel;
using System.Globalization;
using System.IO;
using WideStream.Startup.Resources;
using WideStream.Core.Runtime;
using System.Reflection;

namespace WideStream.Workspace.Forms
{
    public sealed class LanguagesDropDownList : ComboBox, IEditor
    {
        #region Construction
        public LanguagesDropDownList()
        {
            DropDownStyle = ComboBoxStyle.DropDownList;
         }
        #endregion


        #region Methods
        public void SetStandardValues(object value)
        {
            Items.Clear();
            Items.Add(new ValuePair<String,String>(StringTable.GetString("LanguagesDropDownList.AutoSelectLanguage"), "neutral"));
            
            foreach (string s in typeof(LanguagesDropDownList).Assembly.GetManifestResourceNames())
                if (s.EndsWith(".wsresx"))
                {
                    string str = s.Replace("WideStream.Startup.Resources.Translations.", String.Empty).
                        Replace(".wsresx", String.Empty);
                    CultureInfo ci = CultureInfo.GetCultureInfoByIetfLanguageTag(str);
                    Items.Add(new ValuePair<String, String>(ci.DisplayName, str));
                }

            PopulateList(new DirectoryInfo(Application.StartupPath));
            PopulateList(new DirectoryInfo(Path.Combine(Application.StartupPath, "Resources")));
        }


        private void PopulateList(DirectoryInfo dir)
        {
            if (!dir.Exists)
                return;

            foreach (FileInfo fi in dir.GetFiles("*.wsresx"))
            {
                string lang = fi.Name.Replace(fi.Extension, String.Empty);

                try
                {
                    CultureInfo ci = CultureInfo.GetCultureInfoByIetfLanguageTag(lang);
                    Items.Add(new ValuePair<String,String>(ci.DisplayName, lang));
                }
                catch (ArgumentException) { }
            }
        }


        protected override void OnSelectedIndexChanged(EventArgs e)
        {
            IsDirty = true;
            OnValueChanged(new EditorEventArgs(Value));
            base.OnSelectedIndexChanged(e);
        }
        #endregion


        #region Properties
        public object Value
        {
            get
            {
               return ((ValuePair<String,String>)SelectedItem).Value;
            }
            set
            {
                if (value != null && value as String != "neutral")
                {
                    try
                    {
                        Text = CultureInfo.GetCultureInfoByIetfLanguageTag(value.ToString()).DisplayName;
                    }
                    catch (ArgumentException) { }
                }
                else
                    SelectedIndex = 0;

                IsDirty = false;
            }
        }

        public PropertyInfo RelatedProperty { get; set; }
        
        public bool IsDirty { get; private set; }
        #endregion


        #region Events
        public event EventHandler<EditorEventArgs> ValueChanged;
        private void OnValueChanged(EditorEventArgs e)
        {
            EventHandler<EditorEventArgs> h = ValueChanged;

            if (h != null)
                h(this, e);
        }
        #endregion
    }
}
