﻿using System;
using System.Windows.Forms;
using WideStream.Workspace.ComponentModel;
using WideStream.Startup.Resources;

namespace WideStream.Workspace.State
{
    public static class LocalizationManager
    {
        #region Construction
        private const char PREFIX = '$';
        #endregion


        #region Methods
        public static void Localize(IControl cont)
        {
            if (cont.DisplayControl != null)
                foreach (DataGridViewColumn c in cont.DisplayControl.Columns)
                    CheckText(c);

            if (cont.ColumnsMenu != null)
                ScanMenuItems(cont.ColumnsMenu.Items);

            if (cont.ContextMenu != null)
                ScanMenuItems(cont.ContextMenu.Items);

            if (cont.MainMenu != null)
                ScanMenuItems(cont.MainMenu.Items);

            if (cont.ToolBar != null)
                ScanMenuItems(cont.ToolBar.Items);
        }


        private static void ScanMenuItems(ToolStripItemCollection items)
        {
            foreach (ToolStripItem i in items)
            {
                CheckText(i);

                if (i is ToolStripMenuItem)
                    ScanMenuItems(((ToolStripMenuItem)i).DropDownItems);
            }
        }


        private static void CheckText(DataGridViewColumn col)
        {
            if (col.HeaderText.Length > 0 && col.HeaderText[0] == PREFIX)
                col.HeaderText = StringTable.GetString(col.HeaderText);
        }


        private static void CheckText(ToolStripItem mi)
        {
            if (mi.Text.Length > 0 && mi.Text[0] == PREFIX)
                mi.Text = StringTable.GetString(mi.Text);

            if (!String.IsNullOrEmpty(mi.ToolTipText) && mi.ToolTipText[0] == PREFIX)
                mi.ToolTipText = StringTable.GetString(mi.ToolTipText);
        }
        #endregion
    }
}
