﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WideStream.Workspace.ComponentModel;
using WideStream.Workspace.Configuration;
using WideStream.Configuration;
using System.Windows.Forms;
using WideStream.Startup;

namespace WideStream.Workspace.State
{
    public static class ThemeManager
    {
        #region Methods
        public static void ApplyTheme(IControl cont)
        {
            cont.ToolBar.GripStyle = ToolbarPosition == StripPosition.Floating ?
                ToolStripGripStyle.Visible : ToolStripGripStyle.Hidden;
            cont.MainMenu.GripStyle = MenuPosition == StripPosition.Floating ?
                ToolStripGripStyle.Visible : ToolStripGripStyle.Hidden;
            ChangeRenderMode(cont, Theme == VisualTheme.ApplicationControlled ?
                ToolStripRenderMode.ManagerRenderMode : ToolStripRenderMode.System);
        }


        private static void ChangeRenderMode(IControl cont, ToolStripRenderMode mode)
        {
            cont.MainMenu.RenderMode = mode;
            cont.ToolBar.RenderMode = mode;
            cont.ContextMenu.RenderMode = mode;
            cont.ColumnsMenu.RenderMode = mode;

            if (Program.Environment != null)
            {
                var cnt = Program.Environment.MainForm.ToolContainer;
                cnt.TopToolStripPanel.RenderMode = mode;
                cnt.BottomToolStripPanel.RenderMode = mode;
                cnt.LeftToolStripPanel.RenderMode = mode;
                cnt.RightToolStripPanel.RenderMode = mode;
            }
        }
        #endregion


        #region Properties
        private static StripPosition? _toolbarPosition;
        public static StripPosition ToolbarPosition
        {
            get
            {
                if (_toolbarPosition == null)
                    lock (typeof(ThemeManager))
                        if (_toolbarPosition == null)
                            _toolbarPosition = Config.
                                Query<WorkspaceOptions>().ToolbarPosition;

                return _toolbarPosition.Value;
            }
        }


        private static StripPosition? _menuPosition;
        public static StripPosition MenuPosition
        {
            get
            {
                if (_menuPosition == null)
                    lock (typeof(ThemeManager))
                        if (_menuPosition == null)
                            _menuPosition = Config.
                                Query<WorkspaceOptions>().MenuPosition;

                return _menuPosition.Value;
            }
        }


        private static VisualTheme? _theme;
        public static VisualTheme Theme
        {
            get
            {
                if (_theme == null)
                    lock (typeof(ThemeManager))
                        if (_theme == null)
                            _theme = Config.Query<WorkspaceOptions>().Theme;

                return _theme.Value;
            }
        }
        #endregion
    }
}
